/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#define rip_auth_unconfigured 0
#define rip_auth_text 1
#define rip_auth_md5 2
#define rip_auth_none 3
#define rip_auth_unknown 4

#define ospf_default 0
#define ospf_broadcast 1
#define ospf_nonbroadcast 2
#define ospf_p2p 3
#define ospf_p2multi 4
#define ospf_p2multinon 5

struct interfaceConfig
{
	// Standard...
	char name[32];
	char description[128];
	char ipAddress[32];
	int shutdown;					// true, false

	// Interface Options...
	int proxyArp;					// true, false
	int ntp;						// true, false
	int mop;						// true, false
	int ipUnreachables;				// true, false
	int ipRedirects;				// true, false
	int ipMaskReply;				// true, false
	int ipDirectBroadcast;			// true, false, unconfigured
	int cdp;						// true, false
	
	// RIP...
	int ripAuthMode;				// rip_auth_unconfigured, rip_auth_text...
	char ripKeyChain[32];

	// Access Control Lists
	int accessGroupIn;
	int accessGroupOut;

	// OSPF...
	int ospfAuthentication;			// auth_none...
	char ospfKey[128];
	int ospfKeyEncryption;
	char ospfKeyEncrypted[128];
	int ospfDictionary;				// true, false
	int ospfWeak;					// true, false
	int ospfFloodReduction;			// true, false
	int ospfNetwork;				// ospf_broadcast...

	struct interfaceConfig *next;
};


// Process Interface
void processInterface(char *line, struct ciscoIOSConfig *config, FILE *input, int lineSize, const int debugMode, FILE *dictionaryFile)
{
	// Variables
	struct interfaceConfig *interfacePointer;
	struct ciscoCommand command;
	char tempString[lineSize];
	int setting;
	int partPointer;

	// Debug
	if (debugMode == true)
	{
		printf("Interface Line: %s\n", line);
	}

	// First interface
	if (config->interface == 0)
	{
		// create struct
		config->interface = malloc(sizeof(struct interfaceConfig));

		// Pointers
		interfacePointer = config->interface;
	}
	else
	{
		// Find last interface
		interfacePointer = config->interface;
		while (interfacePointer->next != 0)
			interfacePointer = interfacePointer->next;

		// Create struct
		interfacePointer->next = malloc(sizeof(struct interfaceConfig));

		// Pointers
		interfacePointer = interfacePointer->next;
	}

	// Init
	command = splitLine(line);
	memset(interfacePointer, 0, sizeof(struct interfaceConfig));
	strncpy(interfacePointer->name, command.part[1], sizeof(interfacePointer->name));
	strcpy(interfacePointer->ipAddress, "None");
	interfacePointer->proxyArp = true;
	interfacePointer->ntp = true;
	if (strncmp(interfacePointer->name, "Ethernet", 8) == 0)
		interfacePointer->mop = true;
	else
		interfacePointer->mop = false;
	interfacePointer->ipUnreachables = true;
	interfacePointer->ipRedirects = true;
	interfacePointer->ipMaskReply = false;
	if ((config->versionMajor < 11) || ((config->versionMajor == 11) && (config->versionMinor < 4)))
		interfacePointer->ipDirectBroadcast = true;
	else
		interfacePointer->ipDirectBroadcast = false;
	interfacePointer->cdp = true;
	interfacePointer->shutdown = false;
	interfacePointer->ripAuthMode = rip_auth_unconfigured;
	interfacePointer->ospfAuthentication = auth_none;
	interfacePointer->ospfKeyEncryption = encrypt_none;
	interfacePointer->ospfDictionary = false;
	interfacePointer->ospfWeak = false;
	if ((config->versionMajor < 12) || ((config->versionMajor == 12) && (config->versionMinor < 1)))
		interfacePointer->ospfFloodReduction = false;
	else
		interfacePointer->ospfFloodReduction = unconfigured;
	interfacePointer->ospfNetwork = ospf_default;

	// Loop through the interface lines
	readLine(input, line, lineSize);
	while ((feof(input) == 0) && (line[0] == ' '))
	{
		// Debug
		if (debugMode == true)
		{
			printf("Interface Line: %s\n", line);
		}

		// Init
		command = splitLine(line);

		// If no
		if (strcmp(command.part[0], "no") == 0)
		{
			setting = false;
			partPointer = 1;
		}
		else
		{
			setting = true;
			partPointer = 0;
		}

		// Description
		if ((strcmp(command.part[0], "description") == 0) && (command.parts > 1))
		{
			int loop;
			strncpy(interfacePointer->description, command.part[1], sizeof(interfacePointer->description));
			tempString[0] = 0;
			for (loop = 2; loop < command.parts; loop++)
			{
				sprintf(tempString, "%s %s", interfacePointer->description, command.part[loop]);
				strncpy(interfacePointer->description, tempString, sizeof(interfacePointer->description));
			}
		}

		// IP
		else if (strcmp(command.part[partPointer], "ip") == 0)
		{
			partPointer++;

			// IP Address
			if ((strcmp(command.part[partPointer], "address") == 0) && (setting == true) && (command.parts > 2))
			{
				int loop;
				strncpy(interfacePointer->ipAddress, command.part[2], sizeof(interfacePointer->ipAddress));
				tempString[0] = 0;
				for (loop = 3; loop < command.parts; loop++)
				{
					sprintf(tempString, "%s %s", interfacePointer->ipAddress, command.part[loop]);
					strncpy(interfacePointer->ipAddress, tempString, sizeof(interfacePointer->ipAddress));
				}
			}

			// Access Group
			else if (strcmp(command.part[partPointer], "access-group") == 0)
			{
				// In
				if (strcmp(command.part[partPointer + 2], "in") == 0)
					interfacePointer->accessGroupIn = atoi(command.part[partPointer + 1]);
				else if (strcmp(command.part[partPointer + 2], "out") == 0)
					interfacePointer->accessGroupOut = atoi(command.part[partPointer + 1]);
			}

			// ARP Proxy
			else if (strcmp(command.part[partPointer], "proxy-arp") == 0)
				interfacePointer->proxyArp = setting;

			// IP Unreachables
			else if (strcmp(command.part[partPointer], "unreachables") == 0)
				interfacePointer->ipUnreachables = setting;

			// IP Redirects
			else if (strcmp(command.part[partPointer], "redirects") == 0)
				interfacePointer->ipRedirects = setting;

			// IP Mask-reply
			else if (strcmp(command.part[partPointer], "mask-reply") == 0)
				interfacePointer->ipMaskReply = setting;

			// IP Direct Broadcast
			else if (strcmp(command.part[partPointer], "directed-broadcast") == 0)
				interfacePointer->ipDirectBroadcast = setting;

			// RIP Authentication
			else if ((strcmp(command.part[partPointer], "rip") == 0) && (strcmp(command.part[partPointer + 1], "authentication") == 0))
			{
				// Mode
				if (strcmp(command.part[partPointer + 2], "mode") == 0)
				{
					if (setting == false)
						interfacePointer->ripAuthMode = rip_auth_none;
					else if (strcmp(command.part[partPointer + 3], "md5") == 0)
						interfacePointer->ripAuthMode = rip_auth_md5;
					else if (strcmp(command.part[partPointer + 3], "text") == 0)
						interfacePointer->ripAuthMode = rip_auth_text;
					else
						interfacePointer->ripAuthMode = rip_auth_unknown;
				}

				// Key Chain
				else if (strcmp(command.part[partPointer + 2], "key-chain") == 0)
					strncpy(interfacePointer->ripKeyChain, command.part[partPointer + 3], sizeof(interfacePointer->ripAuthMode) - 1);
			}

			else if (strcmp(command.part[partPointer], "ospf") == 0)
			{
				// OSPF Clear-text Authentication
				if (strcmp(command.part[partPointer + 1], "authentication-key") == 0)
				{
					interfacePointer->ospfAuthentication = auth_clear;
					if (strcmp(command.part[partPointer + 2], "7") == 0)
					{
						strncpy(interfacePointer->ospfKeyEncrypted, command.part[partPointer + 3], sizeof(interfacePointer->ospfKeyEncrypted) - 1);
						interfacePointer->ospfKeyEncryption = encrypt_type7;
						if (password7(command.part[partPointer + 3], interfacePointer->ospfKey, sizeof(interfacePointer->ospfKey), debugMode) != 0)
							strcpy(interfacePointer->ospfKey, "<unknown>");
						if (simplePassword(interfacePointer->ospfKey, dictionaryFile) == true)
						{
							config->simplePasswords++;
							interfacePointer->ospfDictionary = true;
						}
						if (passwordStrength(interfacePointer->ospfKey) == false)
						{
							config->passwordStrengths++;
							interfacePointer->ospfWeak = true;
						}
					}
					else if (strcmp(command.part[partPointer + 2], "5") == 0)
					{
						strncpy(interfacePointer->ospfKeyEncrypted, command.part[partPointer + 3], sizeof(interfacePointer->ospfKeyEncrypted) - 1);
						interfacePointer->ospfKeyEncryption = encrypt_md5;
						strcpy(interfacePointer->ospfKey, "unknown");
					}
					else if (strcmp(command.part[partPointer + 2], "0") == 0)
					{
						strncpy(interfacePointer->ospfKey, command.part[partPointer + 3], sizeof(interfacePointer->ospfKey) - 1);
						if (simplePassword(interfacePointer->ospfKey, dictionaryFile) == true)
						{
							config->simplePasswords++;
							interfacePointer->ospfDictionary = true;
						}
						if (passwordStrength(interfacePointer->ospfKey) == false)
						{
							config->passwordStrengths++;
							interfacePointer->ospfWeak = true;
						}
					}
					else
					{
						strncpy(interfacePointer->ospfKey, command.part[partPointer + 2], sizeof(interfacePointer->ospfKey) - 1);
						if (simplePassword(interfacePointer->ospfKey, dictionaryFile) == true)
						{
							config->simplePasswords++;
							interfacePointer->ospfDictionary = true;
						}
						if (passwordStrength(interfacePointer->ospfKey) == false)
						{
							config->passwordStrengths++;
							interfacePointer->ospfWeak = true;
						}
					}
				}

				// OSPF MD5 Authentication
				else if (strcmp(command.part[partPointer + 1], "message-digest-key") == 0)
				{
					interfacePointer->ospfAuthentication = auth_md5;
					strncpy(interfacePointer->ospfKey, command.part[command.parts - 1], sizeof(interfacePointer->ospfKey) - 1);
					sprintf(tempString, "ospf-int-%s", interfacePointer->name);
						addJohnPassword(config, tempString, interfacePointer->ospfKey);
				}

				// OSPF Flood reduction
				else if (strcmp(command.part[partPointer + 1], "flood-reduction") == 0)
					interfacePointer->ospfFloodReduction = setting;

				// OSPF Network
				else if (strcmp(command.part[partPointer + 1], "network") == 0)
				{
					if (strcmp(command.part[partPointer + 2], "broadcast") == 0)
						interfacePointer->ospfNetwork = ospf_broadcast;
					else if (strcmp(command.part[partPointer + 2], "non-broadcast") == 0)
						interfacePointer->ospfNetwork = ospf_nonbroadcast;
					else if (strcmp(command.part[partPointer + 2], "point-to-point") == 0)
						interfacePointer->ospfNetwork = ospf_p2p;
					else if (strcmp(command.part[partPointer + 2], "point-to-multipoint") == 0)
					{
						interfacePointer->ospfNetwork = ospf_p2multi;
						if (command.parts > partPointer + 3)
						{
							if (strcmp(command.part[partPointer + 3], "non-broadcast") == 0)
								interfacePointer->ospfNetwork = ospf_p2multinon;
						}
					}
					if (setting == false)
						interfacePointer->ospfNetwork = ospf_default;
				}
			} 
		}

		// MOP
		else if ((strcmp(command.part[partPointer], "mop") == 0) && (strcmp(command.part[partPointer + 1], "enable") == 0))
			interfacePointer->mop = setting;

		// CDP
		else if ((strcmp(command.part[partPointer], "cdp") == 0) && (strcmp(command.part[partPointer + 1], "enable") == 0))
			interfacePointer->cdp = setting;

		// NTP
		else if ((strcmp(command.part[0], "ntp") == 0) && (strcmp(command.part[1], "disable") == 0))
			interfacePointer->ntp = false;

		// Shutdown
		else if (strcmp(command.part[0], "shutdown") == 0)
			interfacePointer->shutdown = true;

		// Read next line
		readLine(input, line, lineSize);
	}
}

