/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define line_console 0
#define line_vty 1
#define line_aux 2
#define line_tty 3


// Line
struct lineConfig
{
	int lineNumberStart;
	int lineNumberEnd;
	int lineType;					// line_console...
	int lineLogin;					// true, false, unconfigured
	int lineSpeed;					// Default for console in 9600
	int execTimeout;				// Exec timeout in seconds, 0 = no timeout
	int absoluteTimeout;
	int sessionTimeout;
	int timeout;
	int exec;						// true or false
	int callback;					// true or false
	char password[128];
	char passwordEncrypted[128];
	int encryption;					// encrypt_type7, encrypt_md5...
	int weak;
	int dictionary;
	struct lineConfig *next;
};


// Process Line
void processLine(char *line, struct ciscoIOSConfig *config, FILE *input, int lineSize, const int debugMode, FILE *dictionaryFile)
{
	// Variables
	struct lineConfig *linePointer;
	struct ciscoCommand command;
	fpos_t filePosition;

	// Debug
	if (debugMode == true)
	{
		printf("Line Line: %s\n", line);
	}

	// Get last line and create new struct...
	if (config->line == 0)
	{
		config->line = malloc(sizeof(struct lineConfig));
		linePointer = config->line;
	}
	else
	{
		linePointer = config->line;
		while (linePointer->next != 0)
			linePointer = linePointer->next;
		linePointer->next = malloc(sizeof(struct lineConfig));
		linePointer = linePointer->next;
	}

	// Init Struct
	memset(linePointer, 0, sizeof(struct lineConfig));
	command = splitLine(line);
	linePointer->next = 0;
	if (strncmp(command.part[1], "tty", 3) == 0)
		linePointer->lineType = line_tty;
	else if (strncmp(command.part[1], "vty", 3) == 0)
		linePointer->lineType = line_vty;
	else if ((strncmp(command.part[1], "aux", 3) == 0) || (strncmp(command.part[1], "auxiliary", 9) == 0))
		linePointer->lineType = line_aux;
	else if ((strncmp(command.part[1], "con", 3) == 0) || (strncmp(command.part[1], "console", 7) == 0))
		linePointer->lineType = line_console;
	linePointer->lineNumberStart = atoi(command.part[2]);
	if (command.parts == 4)
		linePointer->lineNumberEnd = atoi(command.part[3]);
	else
		linePointer->lineNumberEnd = 0;
	linePointer->lineSpeed = 9600;
	linePointer->lineLogin = unconfigured;
	strcpy(linePointer->password, "");
	strcpy(linePointer->passwordEncrypted, "");
	linePointer->encryption = encrypt_none;
	linePointer->weak = false;
	linePointer->dictionary = false;
	linePointer->execTimeout = 0;
	linePointer->absoluteTimeout = 0;
	linePointer->sessionTimeout = 0;
	linePointer->timeout = 0;
	linePointer->exec = true;
	linePointer->callback = false;

	// Loop through the interface lines
	fgetpos(input, &filePosition);
	readLine(input, line, lineSize);
	while ((feof(input) == 0) && (line[0] == ' '))
	{
		// Debug
		if (debugMode == true)
		{
			printf("Line Line: %s\n", line);
		}

		// Init
		command = splitLine(line);

		// Login
		if (strncmp(command.part[0], "login", 5) == 0)
			linePointer->lineLogin = true;

		// No Login
		else if ((strncmp(command.part[0], "no", 2) == 0) && (strncmp(command.part[1], "login", 5) == 0))
			linePointer->lineLogin = false;

		// timeout
		else if (strncmp(command.part[0], "timeout", 7) == 0)
		{
			linePointer->timeout = atoi(command.part[1]) * 60;
			if (command.parts == 3)
				linePointer->timeout = linePointer->timeout + atoi(command.part[2]);
		}

		// No Timeout
		else if ((strncmp(command.part[0], "no", 2) == 0) && (strncmp(command.part[1], "timeout", 7) == 0))
			linePointer->timeout = 0;

		// Exec-timeout
		else if (strncmp(command.part[0], "exec-timeout", 12) == 0)
		{
			linePointer->execTimeout = atoi(command.part[1]) * 60;
			if (command.parts == 3)
				linePointer->execTimeout = linePointer->execTimeout + atoi(command.part[2]);
		}

		// No Exec Timeout
		else if ((strncmp(command.part[0], "no", 2) == 0) && (strncmp(command.part[1], "exec-timeout", 12) == 0))
			linePointer->execTimeout = 0;

		// session-timeout
		else if (strncmp(command.part[0], "session-timeout", 15) == 0)
		{
			linePointer->sessionTimeout = atoi(command.part[1]) * 60;
			if (command.parts == 3)
				linePointer->sessionTimeout = linePointer->sessionTimeout + atoi(command.part[2]);
		}

		// No session Timeout
		else if ((strncmp(command.part[0], "no", 2) == 0) && (strncmp(command.part[1], "session-timeout", 15) == 0))
			linePointer->sessionTimeout = 0;

		// Absolute-timeout
		else if (strncmp(command.part[0], "absolute-timeout", 16) == 0)
		{
			linePointer->absoluteTimeout = atoi(command.part[1]) * 60;
			if (command.parts == 3)
				linePointer->absoluteTimeout = linePointer->absoluteTimeout + atoi(command.part[2]);
		}

		// No Absolute Timeout
		else if ((strncmp(command.part[0], "no", 2) == 0) && (strncmp(command.part[1], "absolute-timeout", 16) == 0))
			linePointer->absoluteTimeout = 0;

		// Exec
		else if (strncmp(command.part[0], "exec", 4) == 0)
			linePointer->exec = true;

		// No Exec
		else if ((strncmp(command.part[0], "no", 2) == 0) && (strncmp(command.part[1], "exec", 4) == 0))
			linePointer->exec = false;

		// Exec
		else if (strncmp(command.part[0], "callback", 8) == 0)
			linePointer->callback = true;

		// Password
		else if (strncmp(command.part[0], "password", 8) == 0)
		{
			switch(command.part[1][0])
			{
				case '7':
					linePointer->encryption = encrypt_type7;
					if (password7(command.part[command.parts - 1], linePointer->password, sizeof(linePointer->password), debugMode) != 0)
						strcpy(linePointer->password, "<unknown>");
					strncpy(linePointer->passwordEncrypted, command.part[command.parts - 1], sizeof(linePointer->passwordEncrypted));
					break;
				case '5':
					linePointer->encryption = encrypt_md5;
					strcpy(linePointer->password, "<unknown>");
					strncpy(linePointer->passwordEncrypted, command.part[command.parts - 1], sizeof(linePointer->passwordEncrypted));
					addJohnPassword(config, "line", linePointer->passwordEncrypted);
					break;
				case '0':
					linePointer->encryption = encrypt_none;
					strncpy(linePointer->password, command.part[command.parts - 1], sizeof(linePointer->password));
					strcpy(linePointer->passwordEncrypted, "");
					break;
				default:
					if (command.parts == 2)
					{
						linePointer->encryption = encrypt_none;
						strncpy(linePointer->password, command.part[command.parts - 1], sizeof(linePointer->password));
						strcpy(linePointer->passwordEncrypted, "");
					}
					else
					{
						linePointer->encryption = encrypt_unknown;
						strcpy(linePointer->password, "<unknown>");
						strncpy(linePointer->passwordEncrypted, command.part[command.parts - 1], sizeof(linePointer->passwordEncrypted));
					}
					break;
			}
			if ((strcmp(linePointer->password, "<unknown>") != 0) && (strlen(linePointer->password) != 0))
			{
				if (simplePassword(linePointer->password, dictionaryFile) == true)
				{
					config->simplePasswords++;
					linePointer->dictionary = true;
				}
				if (passwordStrength(linePointer->password) == false)
				{
					config->passwordStrengths++;
					linePointer->weak = true;
				}
			}
		}

		// Read next line
		fgetpos(input, &filePosition);
		readLine(input, line, lineSize);
	}
	fsetpos(input, &filePosition);
}


