/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#define snmp3_no_auth 0
#define snmp3_auth 1
#define snmp3_priv 2


// SNMP Community
struct snmpCommunity
{
	char community[64];
	int weak;
	int dictionary;
	int readonly;					// true or false
	int disabled;					// true or false
	int accessList;
	char viewName[64];
	struct snmpCommunity *next;
};

// SNMP View Item
struct snmpViewItem
{
	char mibId[128];
	int include;					// true or false
	struct snmpViewItem *next;
};

// SNMP View
struct snmpView
{
	char viewName[64];
	struct snmpViewItem *viewItem;
	struct snmpView *next;
};

// SNMP User
struct snmpUser
{
	char userName[128];
	char authPass[128];
	char privPass[128];
	struct snmpUser *next;
};

// SNMP Group
struct snmpGroup
{
	char groupName[64];
	int snmpSecurity;				// snmp3_no_auth, snmp3_auth, snmp3_priv
	struct snmpUser *user;
	struct snmpGroup *next;
};


// SNMP Host
struct snmpHost
{
	char host[32];
	char community[64];
	int weak;
	int dictionary;
	struct snmpHost *next;
};

// SNMP
struct snmpConfig
{
	int enabled;					// true or false
	int tftpServerList;
	char contact[64];
	char location[64];
	struct snmpCommunity *community;
	struct snmpGroup *group;
	struct snmpView *view;
	struct snmpTrap *trap;
	struct snmpHost *host;
};


// Process SNMP
void processSnmp(char *line, struct ciscoIOSConfig *config, const int debugMode, FILE *dictionaryFile)
{
	// Variables...
	struct snmpCommunity *communityPointer;
	struct snmpViewItem *snmpViewItemPointer;
	struct snmpView *snmpViewPointer;
	//struct snmpGroup *snmpGroupPointer;
	//struct snmpUser *snmpUserPointer;
	struct snmpTrap *snmpTrapPointer;
	struct snmpHost *snmpHostPointer;
	int tempInt;
	int offset;
	int eol;

	// Debug
	if (debugMode == true)
	{
		printf("SNMP Line: %s\n", line);
	}

	// Check to see if SNMP already exists
	if (config->snmp == 0)
	{
		config->snmp = malloc(sizeof(struct snmpConfig));
		memset(config->snmp, 0, sizeof(struct snmpConfig));

		// Init
		config->snmp->enabled = true;
		config->snmp->tftpServerList = -1;
		strcpy(config->snmp->contact, "");
		strcpy(config->snmp->location, "");
		config->snmp->community = 0;
		config->snmp->group = 0;
		config->snmp->view = 0;
		config->snmp->trap = 0;
		config->snmp->host = 0;
	}

	// snmp contact info?
	if (strncmp(line, "snmp-server contact ", 20) == 0)
		strncpy(config->snmp->contact, line + 20, sizeof(config->snmp->contact));

	// snmp location info?
	else if (strncmp(line, "snmp-server location ", 21) == 0)
		strncpy(config->snmp->location, line + 21, sizeof(config->snmp->location));

	// snmp server disabled?
	else if (strcmp(line, "no snmp-server") == 0)
		config->snmp->enabled = false;

	// tftp server access list?
	else if (strncmp(line, "snmp-server tftp-server-list ", 29) == 0)
		config->snmp->tftpServerList = atoi(line + 29);

	// snmp view?
	else if (strncmp(line, "snmp-server view ", 17) == 0)
	{
		// Find end of view name
		tempInt = nextSpace(line + 17);
		line[17 + tempInt] = 0;
		offset = 18 + tempInt;

		// Is this the first view?
		if (config->snmp->view == 0)
		{
			// create structure...
			config->snmp->view = malloc(sizeof(struct snmpView));
			memset(config->snmp->view, 0, sizeof(struct snmpView));
			snmpViewPointer = config->snmp->view;

			// init
			snmpViewPointer->viewItem = 0;
			strncpy(snmpViewPointer->viewName, line+17, sizeof(snmpViewPointer->viewName));
		}

		else
		{
			// Search for existing view name
			snmpViewPointer = config->snmp->view;
			while ((snmpViewPointer->next != 0) && (strcmp(snmpViewPointer->viewName, line+17) != 0))
				snmpViewPointer = snmpViewPointer->next;

			// Do we need to create a new view?
			if (strcmp(snmpViewPointer->viewName, line+17) != 0)
			{
				// create structure...
				snmpViewPointer->next = malloc(sizeof(struct snmpView));
				memset(snmpViewPointer->next, 0, sizeof(struct snmpView));

				// Sort out pointers
				snmpViewPointer = snmpViewPointer->next;

				// init
				snmpViewPointer->viewItem = 0;
				strncpy(snmpViewPointer->viewName, line+17, sizeof(snmpViewPointer->viewName));
			}
		}

		// Is it the first view item?
		if (snmpViewPointer->viewItem == 0)
		{
			snmpViewPointer->viewItem = malloc(sizeof(struct snmpViewItem));
			memset(snmpViewPointer->viewItem, 0, sizeof(struct snmpViewItem));
			snmpViewItemPointer = snmpViewPointer->viewItem;
		}

		else
		{
			// Find last view item
			snmpViewItemPointer = snmpViewPointer->viewItem;
			while (snmpViewItemPointer->next != 0)
				snmpViewItemPointer = snmpViewItemPointer->next;

			// create view item structure & sort pointers
			snmpViewItemPointer->next = malloc(sizeof(struct snmpViewItem));
			memset(snmpViewItemPointer->next, 0, sizeof(struct snmpViewItem));
			snmpViewItemPointer = snmpViewItemPointer->next;
		}

		// Find end of view item
		tempInt = nextSpace(line + offset);
		line[offset + tempInt] = 0;
		strncpy(snmpViewItemPointer->mibId, line + offset, sizeof(snmpViewItemPointer->mibId));

		// Include?
		if ((strncmp(line + offset + tempInt + 1, "include", 7) == 0) || (strncmp(line + offset + tempInt + 1, "included", 8) == 0))
			snmpViewItemPointer->include = true;
		else
			snmpViewItemPointer->include = false;
	}

	// snmp community?
	else if (strncmp(line, "snmp-server community ", 22) == 0)
	{
		// create snmp community struct
		communityPointer = malloc(sizeof(struct snmpCommunity));
		memset(communityPointer, 0, sizeof(struct snmpCommunity));

		// Sort out Pointers
		communityPointer->next = config->snmp->community;
		config->snmp->community = communityPointer;

		// Init community
		communityPointer->readonly = unconfigured;
		communityPointer->disabled = false;
		communityPointer->accessList = unconfigured;
		communityPointer->weak = false;
		communityPointer->dictionary = false;

		// Get community string
		offset = 22;
		eol = false;
		tempInt = nextSpace(line + offset);
		if (line[offset + tempInt] == 0)
			eol = true;
		else
			line[offset + tempInt] = 0;
		strncpy(communityPointer->community, line + offset, sizeof(communityPointer->community));
		offset = offset + tempInt + 1;

		// Check strength
		if (simplePassword(communityPointer->community, dictionaryFile) == true)
		{
			config->simplePasswords++;
			communityPointer->dictionary = true;
		}
		if (passwordStrength(communityPointer->community) == false)
		{
			config->passwordStrengths++;
			communityPointer->weak = true;
		}

		// Get view, if it exists
		if (eol == false)
		{
			if (strncmp(line + offset, "view ", 5) == 0)
			{
				tempInt = nextSpace(line + offset);
				if (line[offset + tempInt] == 0)
					eol = true;
				else
					line[offset + tempInt] = 0;
				strncpy(communityPointer->viewName, line + offset, sizeof(communityPointer->viewName));
				offset = offset + tempInt + 1;
			}
		}

		// Get access type, if it exists
		if (eol == false)
		{
			if ((strncmp(line + offset, "ro", 2) == 0) || (strncmp(line + offset, "RO", 2) == 0))
			{
				communityPointer->readonly = true;
				if (line[offset+2] == 0)
					eol = true;
				else
					offset = offset + 3;
			}
			else if ((strncmp(line + offset, "rw", 2) == 0) || (strncmp(line + offset, "RW", 2) == 0))
			{
				communityPointer->readonly = false;
				if (line[offset+2] == 0)
					eol = true;
				else
					offset = offset + 3;
			}
		}

		// Get remainder as access-list
		if (eol == false)
		{
			communityPointer->accessList = atoi(line + offset);
		}
	}

	// snmp trap?
	else if (strncmp(line, "snmp-server enable traps ", 25) == 0)
	{
		// create trap structure
		snmpTrapPointer = malloc(sizeof(struct snmpTrap));
		memset(snmpTrapPointer, 0, sizeof(struct snmpTrap));

		// sort out pointers
		snmpTrapPointer->next = config->snmp->trap;
		config->snmp->trap = snmpTrapPointer;

		// get trap(s)
		strncpy(snmpTrapPointer->trap, line + 25, sizeof(snmpTrapPointer->trap));
	}

	// snmp host?
	else if (strncmp(line, "snmp-server host ", 17) == 0)
	{
		// create trap structure
		snmpHostPointer = malloc(sizeof(struct snmpHost));
		memset(snmpHostPointer, 0, sizeof(struct snmpHost));

		// sort out pointers
		snmpHostPointer->next = config->snmp->host;
		config->snmp->host = snmpHostPointer;

		// init
		snmpHostPointer->dictionary = false;
		snmpHostPointer->weak = false;

		// find end of host name
		tempInt = nextSpace(line +17);
		if (line[17 + tempInt] == 0)
			eol = true;
		else
		{
			eol = false;
			line[17 + tempInt] = 0;
		}

		// get host
		strncpy(snmpHostPointer->host, line + 17, sizeof(snmpHostPointer->host));

		// Get community?
		if (eol == false)
		{
			strncpy(snmpHostPointer->community, line + tempInt + 18, sizeof(snmpHostPointer->community));
			// Check strength
			if (simplePassword(snmpHostPointer->community, dictionaryFile) == true)
			{
				config->simplePasswords++;
				snmpHostPointer->dictionary = true;
			}
			if (passwordStrength(snmpHostPointer->community) == false)
			{
				config->passwordStrengths++;
				snmpHostPointer->weak = true;
			}
		}
		else
		{
			config->simplePasswords++;
			snmpHostPointer->dictionary = true;
			config->passwordStrengths++;
			snmpHostPointer->weak = true;
		}
	}
}
