/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


void output_IOSTablePasswords(struct ciscoIOSConfig *config, int dictPassword, int passwords, int reportFormat, FILE *reportFile, int maxSize)
{
	// Variables
	struct ntpKey *ntpKeyPointer;
	struct enablePassword *enablePointer;
	struct usernameList *usernamePointer;
	struct snmpCommunity *communityPointer;
	struct snmpHost *snmpHostPointer;
	struct aaaGroupServer *aaaGroupPointer;
	struct aaaServer *aaaServerPointer;
	struct interfaceConfig *interfaceConfigPointer;
	struct bgpNeighbor *neighborPointer;
	struct keyChain *keyChainPointer;
	struct keyConfig *keyPointer;
	struct lineConfig *linePointer;
	char tempString[maxSize];

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Enable passwords
	enablePointer = config->enable;
	while (enablePointer != 0)
	{
		if (((enablePointer->dictionary == true) && (dictPassword == true)) || ((enablePointer->weak == true) && (dictPassword == false)))
		{
			outputFriendly(enablePointer->password, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sPassword%sEnable%sLevel %d", tablerow_start, tablerow_mid, tablerow_mid, enablePointer->level);
			if (passwords == true)
				fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
		enablePointer = enablePointer->next;
	}

	// Usernames
	usernamePointer = config->username;
	while (usernamePointer != 0)
	{
		if (((usernamePointer->dictionary == true) && (dictPassword == true)) || ((usernamePointer->weak == true) && (dictPassword == false)))
		{
			outputFriendly(usernamePointer->username, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sPassword%sUsers%s%s", tablerow_start, tablerow_mid, tablerow_mid, tempString);
			if (passwords == true)
			{
				outputFriendly(usernamePointer->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
			}
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
		usernamePointer = usernamePointer->next;
	}

	// OSPF
	interfaceConfigPointer = config->interface;
	while (interfaceConfigPointer != 0)
	{
		if (((interfaceConfigPointer->ospfDictionary == true) && (dictPassword == true)) || ((interfaceConfigPointer->ospfWeak == true) && (dictPassword == false)))
		{
			outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sKey%sOSPF%s%s", tablerow_start, tablerow_mid, tablerow_mid, tempString);
			if (passwords == true)
			{
				outputFriendly(interfaceConfigPointer->ospfKey, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
			}
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
		interfaceConfigPointer = interfaceConfigPointer->next;
	}

	// BGP
	if (config->bgp != 0)
	{
		neighborPointer = config->bgp->neighbor;
		while (neighborPointer != 0)
		{
			if (((neighborPointer->dictionary == true) && (dictPassword == true)) || ((neighborPointer->weak == true) && (dictPassword == false)))
			{
				outputFriendly(neighborPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%sPassword%sBGP%sHost %s", tablerow_start, tablerow_mid, tablerow_mid, tempString);
				if (passwords == true)
				{
					outputFriendly(neighborPointer->password, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
				}
				else
					fprintf(reportFile, "%s", tablerow_end);
			}
			neighborPointer = neighborPointer->next;
		}
	}

	// TACACS
	if (config->tacacs != 0)
	{
		if (((config->tacacs->dictionary == true) && (dictPassword == true)) || ((config->tacacs->weak == true) && (dictPassword == false)))
		{
			outputFriendly(config->tacacs->key, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sKey%sTACACS%sN/A", tablerow_start, tablerow_mid, tablerow_mid);
			if (passwords == true)
			{
				fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
			}
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
	}

	// AAA
	if (config->aaa != 0)
	{
		aaaGroupPointer = config->aaa->groupServer;
		while (aaaGroupPointer != 0)
		{
			aaaServerPointer = aaaGroupPointer->server;
			while (aaaServerPointer != 0)
			{
				if (((aaaServerPointer->dictionary == true) && (dictPassword == true)) || ((aaaServerPointer->weak == true) && (dictPassword == false)))
				{
					outputFriendly(aaaGroupPointer->username, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%sKey%sAAA%s%s (Host: ", tablerow_start, tablerow_mid, tablerow_mid, tempString);
					outputFriendly(aaaServerPointer->server, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s)", tempString);
					if (passwords == true)
					{
						outputFriendly(aaaServerPointer->key, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
					}
					else
						fprintf(reportFile, "%s", tablerow_end);
				}
				aaaServerPointer = aaaServerPointer->next;
			}
			aaaGroupPointer = aaaGroupPointer->next;
		}
	}

	// NTP
	if (config->ntp != 0)
	{
		ntpKeyPointer = config->ntp->key;
		while (ntpKeyPointer != 0)
		{
			if (((ntpKeyPointer->dictionary == true) && (dictPassword == true)) || ((ntpKeyPointer->weak == true) && (dictPassword == false)))
			{
				outputFriendly(ntpKeyPointer->key, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%sKey%sNTP%sN/A", tablerow_start, tablerow_mid, tablerow_mid);
				if (passwords == true)
					fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
				else
					fprintf(reportFile, "%s", tablerow_end);
			}
			ntpKeyPointer = ntpKeyPointer->next;
		}
	}

	// SNMP
	if (config->snmp != 0)
	{
		communityPointer = config->snmp->community;
		while (communityPointer != 0)
		{
			if (((communityPointer->dictionary == true) && (dictPassword == true)) || ((communityPointer->weak == true) && (dictPassword == false)))
			{
				fprintf(reportFile, "%sCommunity%sSNMP%s(", tablerow_start, tablerow_mid, tablerow_mid);
				if (communityPointer->readonly == true)
					fprintf(reportFile, "read-only");
				else
					fprintf(reportFile, "read/write");
				if (passwords == true)
				{
					outputFriendly(communityPointer->community, tempString, maxSize, reportFormat);
					fprintf(reportFile, ")%s%s%s", tablerow_mid, tempString, tablerow_end);
				}
				else
					fprintf(reportFile, ")%s", tablerow_end);
			}
			communityPointer = communityPointer->next;
		}
		snmpHostPointer = config->snmp->host;
		while (snmpHostPointer != 0)
		{
			if (((snmpHostPointer->dictionary == true) && (dictPassword == true)) || ((snmpHostPointer->weak == true) && (dictPassword == false)))
			{
				outputFriendly(snmpHostPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%sCommunity%sSNMP%sHost: %s", tablerow_start, tablerow_mid, tablerow_mid, tempString);
				if (passwords == true)
				{
					outputFriendly(snmpHostPointer->community, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
				}
				else
					fprintf(reportFile, "%s", tablerow_end);
			}
			snmpHostPointer = snmpHostPointer->next;
		}
	}

	// Keys
	keyChainPointer = config->chain;
	while (keyChainPointer != 0)
	{
		keyPointer = keyChainPointer->key;
		while (keyPointer != 0)
		{
			if (((keyPointer->dictionary == true) && (dictPassword == true)) || ((keyPointer->weak == true) && (dictPassword == false)))
			{
				fprintf(reportFile, "%sKey%sKey Chain%sKey No.: %d", tablerow_start, tablerow_mid, tablerow_mid, keyPointer->keyNumber);
				if (passwords == true)
				{
					outputFriendly(keyPointer->key, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
				}
				else
					fprintf(reportFile, "%s", tablerow_end);
			}
			keyPointer = keyPointer->next;
		}
		keyChainPointer = keyChainPointer->next;
	}

	// FTP
	if (config->ftp != 0)
	{
		if (((config->ftp->dictionary == true) && (dictPassword == true)) || ((config->ftp->weak == true) && (dictPassword == false)))
		{
			outputFriendly(config->ftp->username, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sPassword%sFTP%s%s", tablerow_start, tablerow_mid, tablerow_mid, tempString);
			if (passwords == true)
			{
				outputFriendly(config->ftp->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
			}
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
	}

	// Line
	linePointer = config->line;
	while (linePointer != 0)
	{
		if (((linePointer->dictionary == true) && (dictPassword == true)) || ((linePointer->weak == true) && (dictPassword == false)))
		{
			fprintf(reportFile, "%sPassword%sLine%sLine No.: %d", tablerow_start, tablerow_mid, tablerow_mid, linePointer->lineNumberStart);
			if (linePointer->lineNumberEnd > 0)
				fprintf(reportFile, " - %d", linePointer->lineNumberEnd);
			if (passwords == true)
			{
				outputFriendly(linePointer->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_mid, tempString, tablerow_end);
			}
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
		linePointer = linePointer->next;
	}
}


void output_IOSLinePasswords(struct ciscoIOSConfig *config, int dictPassword, int passwords, int reportFormat, FILE *reportFile, int maxSize)
{
	// Variables
	struct ntpKey *ntpKeyPointer;
	struct enablePassword *enablePointer;
	struct usernameList *usernamePointer;
	struct snmpCommunity *communityPointer;
	struct snmpHost *snmpHostPointer;
	struct aaaGroupServer *aaaGroupPointer;
	struct aaaServer *aaaServerPointer;
	struct interfaceConfig *interfaceConfigPointer;
	struct bgpNeighbor *neighborPointer;
	struct keyChain *keyChainPointer;
	struct keyConfig *keyPointer;
	struct lineConfig *linePointer;
	char tempString[maxSize];

	// Enable
	enablePointer = config->enable;
	while (enablePointer != 0)
	{
		if (((enablePointer->dictionary == true) && (dictPassword == true)) || ((enablePointer->weak == true) && (dictPassword == false)))
		{
			outputFriendly(enablePointer->password, tempString, maxSize, reportFormat);
			fprintf(reportFile, "The enable password (level %d) was ", enablePointer->level);
			if (passwords == true)
				fprintf(reportFile, "%s.", tempString);
			else
			{
				if (dictPassword == true)
					fprintf(reportFile, "dictionary-based.");
				else
					fprintf(reportFile, "weak.");
			}
		}
		enablePointer = enablePointer->next;
	}

	// Username
	usernamePointer = config->username;
	while (usernamePointer != 0)
	{
		if (((usernamePointer->dictionary == true) && (dictPassword == true)) || ((usernamePointer->weak == true) && (dictPassword == false)))
		{
			outputFriendly(usernamePointer->username, tempString, maxSize, reportFormat);
			fprintf(reportFile, "The user %s had a ", tempString);
			if (passwords == true)
			{
				outputFriendly(usernamePointer->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "password %s.", tempString);
			}
			else
			{
				if (dictPassword == true)
					fprintf(reportFile, "dictionary-based password.");
				else
					fprintf(reportFile, "weak password.");
			}
		}
		usernamePointer = usernamePointer->next;
	}

	// OSPF
	interfaceConfigPointer = config->interface;
	while (interfaceConfigPointer != 0)
	{
		if (((interfaceConfigPointer->ospfDictionary == true) && (dictPassword == true)) || ((interfaceConfigPointer->ospfWeak == true) && (dictPassword == false)))
		{
			outputFriendly(interfaceConfigPointer->name, tempString, maxSize, reportFormat);
			fprintf(reportFile, "The network interface %s was configured with ", tempString);
			if (passwords == true)
			{
				outputFriendly(interfaceConfigPointer->ospfKey, tempString, maxSize, reportFormat);
				fprintf(reportFile, "the OSPF key %s.", tempString);
			}
			else
			{
				if (dictPassword == true)
					fprintf(reportFile, "a dictionary-based OSPF key.");
				else
					fprintf(reportFile, "a weak OSPF key.");
			}
		}
		interfaceConfigPointer = interfaceConfigPointer->next;
	}

	// BGP
	if (config->bgp != 0)
	{
		neighborPointer = config->bgp->neighbor;
		while (neighborPointer != 0)
		{
			if (((neighborPointer->dictionary == true) && (dictPassword == true)) || ((neighborPointer->weak == true) && (dictPassword == false)))
			{
				outputFriendly(neighborPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "The BGP password for host %s was ", tempString);
				if (passwords == true)
				{
					outputFriendly(neighborPointer->password, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s.", tempString);
				}
				else
				{
					if (dictPassword == true)
						fprintf(reportFile, "dictionary-based.");
					else
						fprintf(reportFile, "weak.");
				}
			}
			neighborPointer = neighborPointer->next;
		}
	}

	// TACACS
	if (config->tacacs != 0)
	{
		if (((config->tacacs->dictionary == true) && (dictPassword == true)) || ((config->tacacs->weak == true) && (dictPassword == false)))
		{
			fprintf(reportFile, "The TACACS key was ");
			if (passwords == true)
			{
				outputFriendly(config->tacacs->key, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s.", tempString);
			}
			else
			{
				if (dictPassword == true)
					fprintf(reportFile, "dictionary-based.");
				else
					fprintf(reportFile, "weak.");
			}
		}
	}

	// AAA
	if (config->aaa != 0)
	{
		aaaGroupPointer = config->aaa->groupServer;
		while (aaaGroupPointer != 0)
		{
			aaaServerPointer = aaaGroupPointer->server;
			while (aaaServerPointer != 0)
			{
				if (((aaaServerPointer->dictionary == true) && (dictPassword == true)) || ((aaaServerPointer->weak == true) && (dictPassword == false)))
				{
					outputFriendly(aaaGroupPointer->username, tempString, maxSize, reportFormat);
					fprintf(reportFile, "The AAA key for user %s on host ", tempString);
					outputFriendly(aaaServerPointer->server, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s was ", tempString);
					if (passwords == true)
					{
						outputFriendly(aaaServerPointer->key, tempString, maxSize, reportFormat);
						fprintf(reportFile, "%s.", tempString);
					}
					else
					{
						if (dictPassword == true)
							fprintf(reportFile, "dictionary-based.");
						else
							fprintf(reportFile, "weak.");
					}
				}
				aaaServerPointer = aaaServerPointer->next;
			}
			aaaGroupPointer = aaaGroupPointer->next;
		}
	}

	// NTP
	if (config->ntp != 0)
	{
		ntpKeyPointer = config->ntp->key;
		while (ntpKeyPointer != 0)
		{
			if (((ntpKeyPointer->dictionary == true) && (dictPassword == true)) || ((ntpKeyPointer->weak == true) && (dictPassword == false)))
			{
				fprintf(reportFile, "The NTP key was ");
				if (passwords == true)
				{
					outputFriendly(ntpKeyPointer->key, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s.", tempString);
				}
				else
				{
					if (dictPassword == true)
						fprintf(reportFile, "dictionary-based.");
					else
						fprintf(reportFile, "weak.");
				}
			}
			ntpKeyPointer = ntpKeyPointer->next;
		}
	}

	// SNMP
	if (config->snmp != 0)
	{
		communityPointer = config->snmp->community;
		while (communityPointer != 0)
		{
			if (((communityPointer->dictionary == true) && (dictPassword == true)) || ((communityPointer->weak == true) && (dictPassword == false)))
			{
				fprintf(reportFile, "The ");
				if (communityPointer->readonly == true)
					fprintf(reportFile, "read-only");
				else
					fprintf(reportFile, "read/write");
				fprintf(reportFile, " SNMP community string was ");
				if (passwords == true)
				{
					outputFriendly(communityPointer->community, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s.", tempString);
				}
				else
				{
					if (dictPassword == true)
						fprintf(reportFile, "dictionary-based.");
					else
						fprintf(reportFile, "weak.");
				}
			}
			communityPointer = communityPointer->next;
		}
		snmpHostPointer = config->snmp->host;
		while (snmpHostPointer != 0)
		{
			if (((snmpHostPointer->dictionary == true) && (dictPassword == true)) || ((snmpHostPointer->weak == true) && (dictPassword == false)))
			{
				outputFriendly(snmpHostPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "The SNMP community string for the host %s was ", tempString);
				if (passwords == true)
				{
					outputFriendly(snmpHostPointer->community, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s.", tempString);
				}
				else
				{
					if (dictPassword == true)
						fprintf(reportFile, "dictionary-based.");
					else
						fprintf(reportFile, "weak.");
				}
			}
			snmpHostPointer = snmpHostPointer->next;
		}
	}

	// Keys
	keyChainPointer = config->chain;
	while (keyChainPointer != 0)
	{
		keyPointer = keyChainPointer->key;
		while (keyPointer != 0)
		{
			if (((keyPointer->dictionary == true) && (dictPassword == true)) || ((keyPointer->weak == true) && (dictPassword == false)))
			{
				fprintf(reportFile, "The key chain key number %d was ", keyPointer->keyNumber);
				if (passwords == true)
				{
					outputFriendly(keyPointer->key, tempString, maxSize, reportFormat);
					fprintf(reportFile, "%s.", tempString);
				}
				else
				{
					if (dictPassword == true)
						fprintf(reportFile, "dictionary-based.");
					else
						fprintf(reportFile, "weak.");
				}
			}
			keyPointer = keyPointer->next;
		}
		keyChainPointer = keyChainPointer->next;
	}

	// FTP
	if (config->ftp != 0)
	{
		if (((config->ftp->dictionary == true) && (dictPassword == true)) || ((config->ftp->weak == true) && (dictPassword == false)))
		{
			outputFriendly(config->ftp->username, tempString, maxSize, reportFormat);
			fprintf(reportFile, "The FTP password for user %s was ", tempString);
			if (passwords == true)
			{
				outputFriendly(config->ftp->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s.", tempString);
			}
			else
			{
				if (dictPassword == true)
					fprintf(reportFile, "dictionary-based.");
				else
					fprintf(reportFile, "weak.");
			}
		}
	}

	// Line
	linePointer = config->line;
	while (linePointer != 0)
	{
		if (((linePointer->dictionary == true) && (dictPassword == true)) || ((linePointer->weak == true) && (dictPassword == false)))
		{
			if (linePointer->lineNumberEnd > 0)
				fprintf(reportFile, "Lines %d to %d", linePointer->lineNumberStart, linePointer->lineNumberEnd);
			else
				fprintf(reportFile, "Line %d", linePointer->lineNumberStart);
			fprintf(reportFile, " had a password ");
			if (passwords == true)
			{
				outputFriendly(linePointer->password, tempString, maxSize, reportFormat);
				fprintf(reportFile, "of %s.", tempString);
			}
			else
			{
				if (dictPassword == true)
					fprintf(reportFile, "that was dictionary-based.");
				else
					fprintf(reportFile, "that was weak.");
			}
		}
		linePointer = linePointer->next;
	}
	output_newPara(reportFile, reportFormat);
}


