/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

void generateNMPReport(int reportFormat, struct ciscoNMPConfig *configNMP, const char *outputFile, int maxSize, int passwords, int fullReport, FILE *cssFile, struct latexConfig *latex, const char *companyName, int abs, int appendix, int glossary, int commonPorts)
{
	// Variables
	FILE *reportFile;
	int tableNum = 1;
	int tempInt;
	int issues = 0;
	char friendlyHostname[maxSize];
	char *appendixFile;
	char tempString[maxSize];
	char tempString2[maxSize];
	struct snmpNMPCommunity *communityNMPPointer;
	struct snmpNMPHost *snmpNMPHostPointer;
	struct snmpTrap *snmpTrapPointer;
	struct hostNMP *hostNMPPointer;
	struct moduleConfig *moduleNMPPointer;
	struct portConfig *portNMPPointer;
	struct vlanConfig *vlanNMPPointer;

	// Issue Counters
	int timeoutIssues = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (outputFile[0] == 0)
		reportFile = stdout;
	else
	{
		reportFile = fopen(outputFile, "w");
		if (reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(configNMP->hostname, friendlyHostname, maxSize, reportFormat);

	// Output file heading (if required)
	output_filestart(reportFile, reportFormat, type_catalyst, friendlyHostname, fullReport, cssFile, latex, companyName, maxSize);

	// Report section 
	if (fullReport == true)
	{

		// Output report section title
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "<hr><h2>Security-Related Issues</h2>\n\n");
				break;
			case output_xml:
				fprintf(reportFile, " <section type=\"security\" title=\"Security-Related Issues\">\n");
				break;
			case output_text:
				fprintf(reportFile, "Security-Related Issues\n~~~~~~~~~~~~~~~~~~~~~~~\n\n");
				break;
			case output_latex:
				fprintf(reportFile, "\\section[AUDIT_%s]\n  {Security Audit of the Cisco Device %s}\n\n", friendlyHostname, friendlyHostname);
				break;
		}

		// Output report introduction
		output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_security);
		output_parseText(report_security_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		// ---------------------------------------
		// Get List of Issues (and generate index)
		// ---------------------------------------

		// Start of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "<ul>\n");
				break;
		}

		// Simple Passwords
		if (configNMP->simplePasswords > 1)
		{
			issues++;
			output_parseIndex(report_dict_title1, reportFile, reportFormat, friendlyHostname, section_security);
		}
		else if (configNMP->simplePasswords > 0)
		{
			issues++;
			output_parseIndex(report_dict_title2, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Password Strength
		if (configNMP->passwordStrengths > 1)
		{
			issues++;
			output_parseIndex(report_strength_title1, reportFile, reportFormat, friendlyHostname, section_security);
		}
		else if (configNMP->passwordStrengths > 0)
		{
			issues++;
			output_parseIndex(report_strength_title2, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// Timeout
		if (configNMP->timeout * 60 > connectionTimeout)
			timeoutIssues++;
		if (timeoutIssues > 0)
		{
			output_parseIndex(report_timeout_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// IP Redirects
		if (configNMP->switchConfig != 0)
		{
			if (configNMP->switchConfig->ipRedirect == true)
			{
				issues++;
				output_parseIndex(report_redirects_title, reportFile, reportFormat, friendlyHostname, section_security);
			}
		}

		// CDP
		if (configNMP->cdp == true)
		{
			issues++;
			output_parseIndex(report_cdp_title, reportFile, reportFormat, friendlyHostname, section_security);
		}

		// IP Unreachables
		if (configNMP->switchConfig != 0)
		{
			if (configNMP->switchConfig->ipUnreachable == true)
			{
				issues++;
				output_parseIndex(report_icmpunreach_title, reportFile, reportFormat, friendlyHostname, section_security);
			}
		}

		// End of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "</ul><br>\n");
				break;
			case output_text:
				fprintf(reportFile, "\n\n");
				break;
		}

		// If No Issues
		if (issues == 0)
			output_parseText(report_noissues, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		output_endsection(reportFile, reportFormat, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		// Dictionary-based passwords / keys
		if (configNMP->simplePasswords > 0)
		{
			if (configNMP->simplePasswords > 1)
				output_parseTitle(report_dict_title1, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			else
				output_parseTitle(report_dict_title2, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			output_parseText(report_dict_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, configNMP->simplePasswords, tableNum);
			if (configNMP->passwordStrengths > 1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_dict_table, &report_dict_heading, tableNum, friendlyHostname);
				output_NMPTablePasswords(configNMP, true, passwords, reportFormat, reportFile, maxSize);
				output_table(reportFile, reportFormat, false, passwords, report_dict_table, &report_dict_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
				output_NMPLinePasswords(configNMP, true, passwords, reportFormat, reportFile, maxSize);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_dict_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, configNMP->simplePasswords, tableNum);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_dict_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, configNMP->simplePasswords, tableNum);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_dict_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, configNMP->simplePasswords, tableNum);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Weak passwords / keys
		if (configNMP->passwordStrengths > 0)
		{
			if (configNMP->passwordStrengths > 1)
				output_parseTitle(report_strength_title1, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			else
				output_parseTitle(report_strength_title2, reportFile, reportFormat, friendlyHostname, rate_high, section_security);
			output_parseText(report_strength_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, configNMP->passwordStrengths, tableNum);
			if (configNMP->passwordStrengths > 1)
			{
				output_table(reportFile, reportFormat, true, passwords, report_strength_table, &report_dict_heading, tableNum, friendlyHostname);
				output_NMPTablePasswords(configNMP, false, passwords, reportFormat, reportFile, maxSize);
				output_table(reportFile, reportFormat, false, passwords, report_strength_table, &report_dict_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
				output_NMPLinePasswords(configNMP, false, passwords, reportFormat, reportFile, maxSize);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_strength_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_strength_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_strength_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_trivial, 0, 0);
			numToWord(reportFile, passwordLength);
			fprintf(reportFile, " characters in length and contain");
			if (passwordUppers == true)
				tempInt = 1;
			else
				tempInt = 0;
			if (passwordLowers == true)
				tempInt++;
			if (passwordEither == true)
				tempInt++;
			if (passwordNumbers == true)
				tempInt++;
			if (passwordSpecials == true)
				tempInt++;
			if (passwordUppers == true)
			{
				fprintf(reportFile, " uppercase");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordLowers == true)
			{
				fprintf(reportFile, " lowercase");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordEither == true)
			{
				fprintf(reportFile, " either uppercase or lowercase characters");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordNumbers == true)
			{
				fprintf(reportFile, " numbers");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (passwordSpecials == true)
			{
				fprintf(reportFile, " special");
				if (tempInt > 2)
					fprintf(reportFile, ",");
				else if (tempInt == 2)
					fprintf(reportFile, " and");
				tempInt--;
			}
			if (((passwordNumbers == true) && (passwordSpecials == false)) || ((passwordEither == false) && (passwordNumbers == false) && (passwordSpecials == false)))
				fprintf(reportFile, ".");
			else
				fprintf(reportFile, "characters.");
			if (reportFormat == output_xml)
				fprintf(reportFile, "</text>\n");
			output_newPara(reportFile, reportFormat);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Timeout
		if (timeoutIssues > 0)
		{
			output_parseTitle(report_timeout_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_timeout_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, timeoutIssues, tableNum);
			if ((connectionTimeout % 60) == 0)
			{
				numToWord(reportFile, (connectionTimeout / 60));
				fprintf(reportFile, " minutes");
			}
			else
			{
				numToWord(reportFile, connectionTimeout / 60);
				fprintf(reportFile, " minutes and ");
				numToWord(reportFile, connectionTimeout % 60);
				fprintf(reportFile, " seconds");
			}
			output_parseText(report_timeout_obs2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, timeoutIssues, tableNum);
			if (timeoutIssues > 1)
			{
				output_table(reportFile, reportFormat, true, false, report_timeout_table, &report_timeout_heading, tableNum, friendlyHostname);
				if (configNMP->timeout * 60 > connectionTimeout)
				{
					fprintf(reportFile, "%sSession Timeout%s", tablerow_start, tablerow_mid);
					numToWord(reportFile, configNMP->timeout);
					if (configNMP->timeout > 1)
						fprintf(reportFile, " minutes%s", tablerow_end);
					else
						fprintf(reportFile, " minute%s", tablerow_end);
				}
				output_table(reportFile, reportFormat, false, false, report_timeout_table, &report_timeout_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
			{
				if (configNMP->timeout * 60 > connectionTimeout)
				{
					fprintf(reportFile, "The session timeout was ");
					numToWord(reportFile, configNMP->timeout);
					if (configNMP->timeout > 1)
						fprintf(reportFile, " minutes.");
					else
						fprintf(reportFile, " minute.");
				}
				output_newPara(reportFile, reportFormat);
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_timeout_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_timeout_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_difficult, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_timeout_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			if ((connectionTimeout % 60) == 0)
			{
				numToWord(reportFile, (connectionTimeout / 60));
				fprintf(reportFile, " minutes");
			}
			else
			{
				numToWord(reportFile, connectionTimeout / 60);
				fprintf(reportFile, " minutes and ");
				numToWord(reportFile, (connectionTimeout % 60));
				fprintf(reportFile, " seconds");
			}
			output_parseText(report_timeout_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// ICMP Redirects
		if (configNMP->switchConfig != 0)
		{
			if (configNMP->switchConfig->ipRedirect == true)
			{
				output_parseTitle(report_redirects_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
				output_parseText(report_redirects_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, -1, tableNum);
				output_endsection(reportFile, reportFormat, section_obs);
				output_parseText(report_redirects_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
				output_endsection(reportFile, reportFormat, section_imp);
				output_parseText(report_redirects_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
				output_endsection(reportFile, reportFormat, section_ease);
				output_parseText(report_redirects_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_rec);
			}
		}

		// CDP
		if (configNMP->cdp == true)
		{
			output_parseTitle(report_cdp_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
			output_parseText(report_cdp_obs, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, -1, 0);
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_cdp_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_cdp_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_cdp_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// IP Unreachables
		if (configNMP->switchConfig != 0)
		{
			if (configNMP->switchConfig->ipUnreachable == true)
			{
				output_parseTitle(report_icmpunreach_title, reportFile, reportFormat, friendlyHostname, rate_low, section_security);
				output_parseText(report_icmpunreach_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, -1, tableNum);
				output_endsection(reportFile, reportFormat, section_obs);
				output_parseText(report_icmpunreach_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_low, 0, 0);
				output_endsection(reportFile, reportFormat, section_imp);
				output_parseText(report_icmpunreach_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
				output_endsection(reportFile, reportFormat, section_ease);
				output_parseText(report_icmpunreach_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_rec);
			}
		}

		// End Section
		output_endsection(reportFile, reportFormat, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------


	// Output configuration section title
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<hr><h2>%s Device Configuration</h2>\n\n", friendlyHostname);
			break;
		case output_text:
			fprintf(reportFile, "%s Device Configuration\n~~~~~~~~~~~~~~~~~~~~~~~~", friendlyHostname);
			for (tempInt = 0; tempInt < strlen(friendlyHostname); tempInt++)
			{
				fprintf(reportFile, "~");
			}
			fprintf(reportFile, "\n\n");
			break;
		case output_xml:
			fprintf(reportFile, " <section type=\"configuration\" title=\"%s Device Configuration\">\n", friendlyHostname);
			break;
		case output_latex:
			fprintf(reportFile, "\\section[AUDIT_%s]\n  {%s Device Configuation}\n\n", friendlyHostname, friendlyHostname);
			break;
	}

	// Output report introduction
	output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_parseText(report_config_cisco_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

	// ------------------------------------------------------
	// Get List of Configuration options (and generate index)
	// ------------------------------------------------------

	// Start of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<ul>\n");
			break;
	}

	// In Configuration Index
	output_parseIndex(settings_general_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (((configNMP->enablePassword != 0) || (configNMP->password != 0)) && (passwords == true))
		output_parseIndex(settings_passwords_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configNMP->switchConfig != 0)
		output_parseIndex(settings_switch_title, reportFile, reportFormat, friendlyHostname, section_config);
	output_parseIndex(settings_cdp_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configNMP->snmp != 0)
		output_parseIndex(settings_snmp_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configNMP->permit != 0)
		output_parseIndex(settings_permit_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configNMP->vlan != 0)
		output_parseIndex(settings_vlan_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configNMP->module != 0)
		output_parseIndex(settings_ports_title, reportFile, reportFormat, friendlyHostname, section_config);

	// End of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "</ul><br>\n");
			break;
		case output_text:
			fprintf(reportFile, "\n\n");
			break;
	}
	output_endsection(reportFile, reportFormat, subsection_end);

	// General Device Settings
	output_parseTitle(settings_general_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	fprintf(reportFile, "%sHostname%s%s %s", tablerow_start, tablerow_mid, friendlyHostname, tablerow_end);
	if (configNMP->version[0] != 0)
		fprintf(reportFile, "%sNMP Version%s%s %s", tablerow_start, tablerow_mid, configNMP->version, tablerow_end);
	if (configNMP->location[0] != 0)
		fprintf(reportFile, "%sLocation%s%s %s", tablerow_start, tablerow_mid, configNMP->location, tablerow_end);
	if (configNMP->contact[0] != 0)
		fprintf(reportFile, "%sContact%s%s %s", tablerow_start, tablerow_mid, configNMP->contact, tablerow_end);
	if (configNMP->coreFile[0] != 0)
		fprintf(reportFile, "%sCore File%s%s %s", tablerow_start, tablerow_mid, configNMP->coreFile, tablerow_end);
	if (configNMP->syslogFile[0] != 0)
		fprintf(reportFile, "%sSyslog File%s%s %s", tablerow_start, tablerow_mid, configNMP->syslogFile, tablerow_end);
	if (configNMP->timeout == 0)
		fprintf(reportFile, "%sIdle Session Timeout%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
	else
	{
		fprintf(reportFile, "%sIdle Session Timeout%s", tablerow_start, tablerow_mid);
		numToWord(reportFile, configNMP->timeout);
		fprintf(reportFile, " minutes%s", tablerow_end);
	}
	if (configNMP->portSecurityAuto == true)
		fprintf(reportFile, "%sPort Security Auto Configure%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sPort Security Auto Configure%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(reportFile, reportFormat, false, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	tableNum++;
	output_endsection(reportFile, reportFormat, subsection_end);
	addAbbreviation("NMP", false);
	addAbbreviation("CDP", false);

	// Passwords
	if ((configNMP->enablePassword != 0) || (configNMP->password != 0))
	{
		output_parseTitle(settings_passwords_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_passwords_title, &settings_passwords_heading, tableNum, friendlyHostname);
		if (configNMP->enablePassword[0] != 0)
		{
			fprintf(reportFile, "%sEnable Password", tablerow_start);
			if (passwords == true)
				fprintf(reportFile, "%s%s %s", tablerow_mid, configNMP->enablePassword, tablerow_end);
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
		if (configNMP->password[0] != 0)
		{
			fprintf(reportFile, "%sLogin Password", tablerow_start);
			if (passwords == true)
				fprintf(reportFile, "%s%s %s", tablerow_mid, configNMP->password, tablerow_end);
			else
				fprintf(reportFile, "%s", tablerow_end);
		}
		output_table(reportFile, reportFormat, false, passwords, settings_passwords_title, &settings_passwords_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Switch Configuration
	if (configNMP->switchConfig != 0)
	{
		output_parseTitle(settings_switch_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_switch_table, &settings_general_heading, tableNum, friendlyHostname);
		if (configNMP->switchConfig->ipRedirect == true)
			fprintf(reportFile, "%sICMP Redirects%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sICMP Redirects%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (configNMP->switchConfig->ipUnreachable == true)
			fprintf(reportFile, "%sICMP Unreachables%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sICMP Unreachables%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		if (configNMP->switchConfig->ipFragmentation == true)
			fprintf(reportFile, "%sIP Fragmentation%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sIP Fragmentation%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_switch_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("IP", false);
		addAbbreviation("ICMP", false);
	}

	// CDP
	output_parseTitle(settings_cdp_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_cdp_table, &settings_general_heading, tableNum, friendlyHostname);
	if (configNMP->cdp == true)
		fprintf(reportFile, "%sCisco Discovery Protocol%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sCisco Discovery Protocol%sGlobally Disabled%s", tablerow_start, tablerow_mid, tablerow_end);
	fprintf(reportFile, "%sCDP Version%s%d%s", tablerow_start, tablerow_mid, configNMP->cdpVersion, tablerow_end);
	fprintf(reportFile, "%sCDP Message Interval%s", tablerow_start, tablerow_mid);
	if (configNMP->cdpInterval % 60 == 0)
	{
		numToWord(reportFile, configNMP->cdpInterval / 60);
		if (configNMP->cdpInterval / 60 > 1)
			fprintf(reportFile, " minutes%s", tablerow_end);
		else
			fprintf(reportFile, " minute%s", tablerow_end);
	}
	else
	{
		numToWord(reportFile, configNMP->cdpInterval / 60);
		if (configNMP->cdpInterval / 60 > 1)
			fprintf(reportFile, " minutes ");
		else
			fprintf(reportFile, " minute ");
		numToWord(reportFile, configNMP->cdpInterval % 60);
		if (configNMP->cdpInterval % 60 > 1)
			fprintf(reportFile, " seconds%s", tablerow_end);
		else
			fprintf(reportFile, " second%s", tablerow_end);
	}
	fprintf(reportFile, "%sCDP TTL%s", tablerow_start, tablerow_mid);
	if (configNMP->cdpHoldtime % 60 == 0)
	{
		numToWord(reportFile, configNMP->cdpHoldtime / 60);
		if (configNMP->cdpHoldtime / 60 > 1)
			fprintf(reportFile, " minutes%s", tablerow_end);
		else
			fprintf(reportFile, " minute%s", tablerow_end);
	}
	else
	{
		numToWord(reportFile, configNMP->cdpHoldtime / 60);
		if (configNMP->cdpHoldtime / 60 > 1)
			fprintf(reportFile, " minutes ");
		else
			fprintf(reportFile, " minute ");
		numToWord(reportFile, configNMP->cdpHoldtime % 60);
		if (configNMP->cdpHoldtime % 60 > 1)
			fprintf(reportFile, " seconds%s", tablerow_end);
		else
			fprintf(reportFile, " second%s", tablerow_end);
	}
	output_table(reportFile, reportFormat, false, passwords, settings_cdp_table, &settings_general_heading, tableNum, friendlyHostname);
	tableNum++;
	addAbbreviation("CDP", false);
	addAbbreviation("TTL", false);

	// SNMP
	if (configNMP->snmp != 0)
	{
		output_parseTitle(settings_snmp_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		
		if (configNMP->snmp->community != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_snmpstring_table, &settings_snmpcomm_heading, tableNum, friendlyHostname);
			communityNMPPointer = configNMP->snmp->community;
			while (communityNMPPointer != 0)
			{
				if (communityNMPPointer->type == snmp_nmp_standard)
					fprintf(reportFile, "%sStandard%s", tablerow_start, tablerow_mid);
				else
					fprintf(reportFile, "%sExtended%s", tablerow_start, tablerow_mid);
				if (passwords == true)
					fprintf(reportFile, "%s %s", communityNMPPointer->community, tablerow_mid);
				switch (communityNMPPointer->access)
				{
					case snmp_nmp_readonly:
						fprintf(reportFile, "Read-Only%s", tablerow_end);
						break;
					case snmp_nmp_readwrite:
						fprintf(reportFile, "Read-Write%s", tablerow_end);
						break;
					case snmp_nmp_readwrite_all:
						fprintf(reportFile, "Read-Write-All%s", tablerow_end);
						break;
				}
				communityNMPPointer = communityNMPPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_snmpstring_table, &settings_snmpcomm_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		if (configNMP->snmp->trap != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_snmptrap_table, &settings_snmptraps_heading, tableNum, friendlyHostname);
			snmpTrapPointer = configNMP->snmp->trap;
			while (snmpTrapPointer != 0)
			{
				if (snmpTrapPointer->enabled == true)
					fprintf(reportFile, "%sEnabled", tablerow_start);
				else
					fprintf(reportFile, "%sDisabled", tablerow_start);
				fprintf(reportFile, "%s%s%s", tablerow_mid, snmpTrapPointer->trap, tablerow_end);
				addAbbreviation(snmpTrapPointer->trap, false);
				snmpTrapPointer = snmpTrapPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_snmptrap_table, &settings_snmptraps_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		if (configNMP->snmp->host != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_snmphost_table, &settings_snmphosts_heading, tableNum, friendlyHostname);
			snmpNMPHostPointer = configNMP->snmp->host;
			while (snmpNMPHostPointer != 0)
			{
				fprintf(reportFile, "%s%s%s", tablerow_start, snmpNMPHostPointer->host, tablerow_mid);
				if (passwords == true)
					fprintf(reportFile, "%s%s", snmpNMPHostPointer->community, tablerow_mid);
				fprintf(reportFile, "%d%s", snmpNMPHostPointer->version, tablerow_mid);
				fprintf(reportFile, "%d%s", snmpNMPHostPointer->port, tablerow_mid);
				fprintf(reportFile, "%s%s", snmpNMPHostPointer->owner, tablerow_mid);
				fprintf(reportFile, "%d%s", snmpNMPHostPointer->index, tablerow_end);
				snmpNMPHostPointer = snmpNMPHostPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_snmphost_table, &settings_snmphosts_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("SNMP", false);
	}

	// Management Service Permit Lists
	if (configNMP->permit != 0)
	{
		output_parseTitle(settings_permit_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_permit_table, &settings_permit_heading, tableNum, friendlyHostname);
		if (configNMP->permit->telnet == true)
			fprintf(reportFile, "%sTelnet%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configNMP->permit->telnet == false)
			fprintf(reportFile, "%sTelnet%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sTelnet%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		if (configNMP->permit->ssh == true)
			fprintf(reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configNMP->permit->ssh == false)
			fprintf(reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sSSH%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		if (configNMP->permit->snmp == true)
			fprintf(reportFile, "%sSNMP%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configNMP->permit->snmp == false)
			fprintf(reportFile, "%sSNMP%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sSNMP%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_permit_table, &settings_permit_heading, tableNum, friendlyHostname);
		tableNum++;
		addAbbreviation("SNMP", false);
		addAbbreviation("SSH", false);

		// Telnet Permission List
		if (configNMP->permit->telnet == true)
		{
			output_table(reportFile, reportFormat, true, passwords, "Telnet permit list", &settings_permitlist_heading, tableNum, friendlyHostname);
			hostNMPPointer = configNMP->permit->telnetHost;
			while (hostNMPPointer != 0)
			{
				outputFriendly(hostNMPPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s%s%s", tablerow_start, tempString, tablerow_mid, hostNMPPointer->netMask, tablerow_end);
				hostNMPPointer = hostNMPPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, "Telnet permit list", &settings_permitlist_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// SSH Permission List
		if (configNMP->permit->ssh == true)
		{
			output_table(reportFile, reportFormat, true, passwords, "SSH permit list", &settings_permitlist_heading, tableNum, friendlyHostname);
			hostNMPPointer = configNMP->permit->sshHost;
			while (hostNMPPointer != 0)
			{
				outputFriendly(hostNMPPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s%s%s", tablerow_start, tempString, tablerow_mid, hostNMPPointer->netMask, tablerow_end);
				hostNMPPointer = hostNMPPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, "SSH permit list", &settings_permitlist_heading, tableNum, friendlyHostname);
			tableNum++;
		}

		// SNMP Permission List
		if (configNMP->permit->snmp == true)
		{
			output_table(reportFile, reportFormat, true, passwords, "SNMP permit list", &settings_permitlist_heading, tableNum, friendlyHostname);
			hostNMPPointer = configNMP->permit->snmpHost;
			while (hostNMPPointer != 0)
			{
				outputFriendly(hostNMPPointer->host, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s%s%s", tablerow_start, tempString, tablerow_mid, hostNMPPointer->netMask, tablerow_end);
				hostNMPPointer = hostNMPPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, "SNMP permit list", &settings_permitlist_heading, tableNum, friendlyHostname);
			tableNum++;
		}
	}

	// VLAN Configuration
	if (configNMP->vlan != 0)
	{
		output_parseTitle(settings_vlan_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_vlan_table, &settings_vlan_heading, tableNum, friendlyHostname);
		vlanNMPPointer = configNMP->vlan;
		while(vlanNMPPointer != 0)
		{
			fprintf(reportFile, "%s%d%s", tablerow_start, vlanNMPPointer->vlan, tablerow_mid);
			fprintf(reportFile, "%s%s", vlanNMPPointer->name, tablerow_mid);
			if (vlanNMPPointer->active == true)
				fprintf(reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(reportFile, "No%s", tablerow_mid);
			fprintf(reportFile, "%s%s", vlanNMPPointer->type, tablerow_mid);
			if (vlanNMPPointer->firewall == -1)
				fprintf(reportFile, "None%s", tablerow_mid);
			else
				fprintf(reportFile, "%d%s", vlanNMPPointer->firewall, tablerow_mid);
			fprintf(reportFile, "%d%s", vlanNMPPointer->said, tablerow_mid);
			fprintf(reportFile, "%d%s", vlanNMPPointer->mtu, tablerow_mid);
			fprintf(reportFile, "%d%s", vlanNMPPointer->spanTreeMaxHops, tablerow_mid);
			fprintf(reportFile, "%d%s", vlanNMPPointer->allRoutesMaxHops, tablerow_end);
			vlanNMPPointer = vlanNMPPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_vlan_table, &settings_vlan_heading, tableNum, friendlyHostname);
		tableNum++;
		addAbbreviation("SAID", false);
		addAbbreviation("VLAN", false);
		addAbbreviation("MTU", false);
	}

	if (configNMP->module != 0)
	{
		output_parseTitle(settings_ports_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		// Modules / Ports
		moduleNMPPointer = configNMP->module;
		while (moduleNMPPointer != 0)
		{
			if (moduleNMPPointer->name[0] == 0)
				sprintf(tempString, "Module %d port configuration", moduleNMPPointer->module);
			else
				sprintf(tempString, "Module %d (%s) port configuration", moduleNMPPointer->module, moduleNMPPointer->name);
			output_table(reportFile, reportFormat, true, passwords, tempString, &settings_ports_heading, tableNum, friendlyHostname);
			portNMPPointer = moduleNMPPointer->ports;
			while (portNMPPointer != 0)
			{
				fprintf(reportFile, "%s%d%s", tablerow_start, portNMPPointer->port, tablerow_mid);
				outputFriendly(portNMPPointer->name, tempString2, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString2, tablerow_mid);
				if (portNMPPointer->enabled == true)
					fprintf(reportFile, "Enabled%s", tablerow_mid);
				else
					fprintf(reportFile, "Disabled%s", tablerow_mid);
				fprintf(reportFile, "%d%s", portNMPPointer->vlan, tablerow_mid);
				switch (portNMPPointer->speed)
				{
					case port_speed_auto:
						fprintf(reportFile, "Auto%s", tablerow_mid);
						break;
					case port_speed_10:
						fprintf(reportFile, "10Mb%s", tablerow_mid);
						break;
					case port_speed_100:
						fprintf(reportFile, "100Mb%s", tablerow_mid);
						break;
					case port_speed_1000:
						fprintf(reportFile, "1000Mb%s", tablerow_mid);
						break;
					case port_speed_auto_10_100:
						fprintf(reportFile, "10/100Mb%s", tablerow_mid);
						break;
				}
				switch (portNMPPointer->duplex)
				{
					case port_duplex_auto:
						fprintf(reportFile, "Auto%s", tablerow_mid);
						break;
					case port_duplex_half:
						fprintf(reportFile, "Half%s", tablerow_mid);
						break;
					case port_duplex_full:
						fprintf(reportFile, "Full%s", tablerow_mid);
						break;
				}
				if (portNMPPointer->portSecurity == true)
					fprintf(reportFile, "On%s", tablerow_mid);
				else
					fprintf(reportFile, "Off%s", tablerow_mid);
				if (portNMPPointer->cdp == true)
					fprintf(reportFile, "On%s", tablerow_mid);
				else
					fprintf(reportFile, "Off%s", tablerow_mid);
				switch (portNMPPointer->trunk)
				{
					case port_trunk_auto:
						fprintf(reportFile, "Auto%s", tablerow_mid);
						break;
					case port_trunk_nonegotiate:
						fprintf(reportFile, "No Negotiate%s", tablerow_mid);
						break;
					case port_trunk_on:
						fprintf(reportFile, "On%s", tablerow_mid);
						break;
					case port_trunk_off:
						fprintf(reportFile, "Off%s", tablerow_mid);
						break;
					case port_trunk_desirable:
						fprintf(reportFile, "Desirable%s", tablerow_mid);
						break;
				}
				if (portNMPPointer->vtp == true)
					fprintf(reportFile, "On%s", tablerow_mid);
				else
					fprintf(reportFile, "Off%s", tablerow_mid);
				switch (portNMPPointer->spantreeGuard)
				{
					case port_guard_none:
						fprintf(reportFile, "Off%s", tablerow_end);
						break;
					case port_guard_default:
						fprintf(reportFile, "On%s", tablerow_end);
						break;
					case port_guard_loop:
						fprintf(reportFile, "On (loop)%s", tablerow_end);
						break;
					case port_guard_root:
						fprintf(reportFile, "On (root)%s", tablerow_end);
						break;
				}
				portNMPPointer = portNMPPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, tempString, &settings_ports_heading, tableNum, friendlyHostname);
			tableNum++;
			moduleNMPPointer = moduleNMPPointer->next;
		}
		addAbbreviation("VLAN", false);
		addAbbreviation("STP", false);
	}

	// End of Config Section
	output_endsection(reportFile, reportFormat, section_end);

	// Appendix
	if (appendix == true)
		generateAppendix(reportFile, reportFormat, friendlyHostname, tableNum, abs, commonPorts, glossary);

	// Output end of file stuff
	output_fileend(reportFormat, reportFile);
}

