/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Ian Ventura-Whiting (Fizz)               *
 *   Fizz@Titania.co.uk                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

void generateSOSReport(int reportFormat, struct screenOSConfig *configSOS, const char *outputFile, int maxSize, int passwords, int fullReport, FILE *cssFile, struct latexConfig *latex, const char *companyName, int abs, int appendix, int glossary, int commonPorts)
{
	// Variables
	FILE *reportFile;
	int tableNum = 1;
	int tempInt;
	int sourceInt;
	int destInt;
	int serviceInt;
	int issues = 0;
	char tempString[maxSize];
	char tempString2[maxSize];
	char friendlyHostname[maxSize];
	char *appendixFile;
	struct adminUsersSOS *userSOSPointer;
	struct authServerSOS *authServerPointer;
	struct interfaceSOS *interfaceSOSPointer;
	struct policySOS *policySOSPointer;
	struct zonePolicyRuleSOS *ruleSOSPointer;
	struct listSOS *listSOSSourcePointer;
	struct listSOS *listSOSDestPointer;
	struct listSOS *listSOSServicePointer;
	struct addressSOS *addressSOSPointer;
	struct mappingSOS *mappingSOSPointer;

	// Issue Counters
	int denyIssue = false;
	int rejectIssue = false;
	int timeoutIssues = 0;
	int policyIssues = 0;

	// Table Pointers
	const char *tablerow_start;
	const char *tablerow_mid;
	const char *tablerow_end;
	switch (reportFormat)
	{
		case output_html:
			tablerow_start = table_html_rowstart;
			tablerow_mid = table_html_rowmid;
			tablerow_end = table_html_rowend;
			break;
		case output_xml:
			tablerow_start = table_xml_rowstart;
			tablerow_mid = table_xml_rowmid;
			tablerow_end = table_xml_rowend;
			break;
		case output_text:
			tablerow_start = table_text_rowstart;
			tablerow_mid = table_text_rowmid;
			tablerow_end = table_text_rowend;
			break;
		case output_latex:
			tablerow_start = table_latex_rowstart;
			tablerow_mid = table_latex_rowmid;
			tablerow_end = table_latex_rowend;
			break;
	}

	// Create / open file for output
	if (outputFile[0] == 0)
		reportFile = stdout;
	else
	{
		reportFile = fopen(outputFile, "w");
		if (reportFile == NULL)
		{
			printf("\n%sERROR: Could not create file '%s'%s\n", COL_RED, outputFile, RESET);
			return;
		}
	}

	// Set output friendly hostname
	outputFriendly(configSOS->hostname, friendlyHostname, maxSize, reportFormat);

	// Output file heading (if required)
	output_filestart(reportFile, reportFormat, type_screenos, friendlyHostname, fullReport, cssFile, latex, companyName, maxSize);

	// Report section 
	if (fullReport == true)
	{
		// Output report section title
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "<hr><h2>Security-Related Issues</h2>\n\n");
				break;
			case output_xml:
				fprintf(reportFile, " <section type=\"security\" title=\"Security-Related Issues\">\n");
				break;
			case output_text:
				fprintf(reportFile, "Security-Related Issues\n~~~~~~~~~~~~~~~~~~~~~~~\n\n");
				break;
			case output_latex:
				fprintf(reportFile, "\\section[AUDIT_%s]\n  {Security Audit of the NetScreen Device %s}\n\n", friendlyHostname, friendlyHostname);
				break;
		}

		// Output report introduction
		output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_security);
		output_parseText(report_security_screenos_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		// ---------------------------------------
		// Get List of Issues (and generate index)
		// ---------------------------------------

		// Start of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "<ul>\n");
				break;
		}

		// Policy Issues
		policySOSPointer = configSOS->policies;
		while (policySOSPointer != 0)
		{
			ruleSOSPointer = policySOSPointer->policyRules;
			while (ruleSOSPointer != 0)
			{
				if ((ruleSOSPointer->permit == sos_permit_reject) && (ruleSOSPointer->disabled == false))
				{
					rejectIssue = true;
					policyIssues++;
				}
				else if ((ruleSOSPointer->permit == sos_permit_permit) && (ruleSOSPointer->disabled == false))
				{
					tempInt = 0;
					listSOSSourcePointer = ruleSOSPointer->source;
					while (listSOSSourcePointer != 0)
					{
						if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
							tempInt++;
						listSOSSourcePointer = listSOSSourcePointer->next;
					}
					listSOSSourcePointer = ruleSOSPointer->dest;
					while (listSOSSourcePointer != 0)
					{
						if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
							tempInt++;
						listSOSSourcePointer = listSOSSourcePointer->next;
					}
					listSOSSourcePointer = ruleSOSPointer->service;
					while (listSOSSourcePointer != 0)
					{
						if (strcasecmp(listSOSSourcePointer->name, "ANY") == 0)
							tempInt++;
						listSOSSourcePointer = listSOSSourcePointer->next;
					}
					if (tempInt > 0)
						policyIssues++;
				}
				if (ruleSOSPointer->next == 0)
				{
					if ((ruleSOSPointer->permit != sos_permit_deny) || (ruleSOSPointer->disabled == true) || (ruleSOSPointer->log == false) || (strcasecmp(ruleSOSPointer->source->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->dest->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->service->name, "ANY") != 0))
					{
						tempInt++;
						denyIssue = true;
					}
				}
				ruleSOSPointer = ruleSOSPointer->next;
			}
			policySOSPointer = policySOSPointer->next;
		}
		if (policyIssues > 0)
		{
			output_parseIndex(report_policy_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Timeout
		if (configSOS->admin != 0)
		{
			if ((configSOS->admin->authTimeout * 60) > connectionTimeout)
				timeoutIssues++;
			authServerPointer = configSOS->authServer;
			while (authServerPointer != 0)
			{
				if ((authServerPointer->timeout * 60 ) > connectionTimeout)
					timeoutIssues++;
				authServerPointer = authServerPointer->next;
			}
		}
		if (timeoutIssues > 0)
		{
			output_parseIndex(report_timeout_title, reportFile, reportFormat, friendlyHostname, section_security);
			issues++;
		}

		// Administrative HTTP Redirect
		if (configSOS->admin != 0)
		{
			if (configSOS->admin->httpRedirect != true)
			{
				output_parseIndex(report_httpred_title, reportFile, reportFormat, friendlyHostname, section_security);
				issues++;
			}
		}
		// Management IP
		if (configSOS->admin != 0)
		{
			if (strcmp(configSOS->admin->adminIP, "0.0.0.0") == 0)
			{
				output_parseIndex(report_mngrip_title, reportFile, reportFormat, friendlyHostname, section_security);
				issues++;
			}
		}

		// End of index
		switch (reportFormat)
		{
			case output_html:
				fprintf(reportFile, "</ul><br>\n");
				break;
			case output_text:
				fprintf(reportFile, "\n\n");
				break;
		}

		// If No Issues
		if (issues == 0)
			output_parseText(report_noissues, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

		output_endsection(reportFile, reportFormat, subsection_end);

		// ----------------------
		// Security Issues Report
		// ----------------------

		// Policy Lists
		if (policyIssues > 0)
		{
			output_parseTitle(report_policy_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_policy_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, policyIssues, tableNum);
			if (policyIssues > 1)
			{
				output_table(reportFile, reportFormat, true, false, report_policy_table, &report_policy_heading, tableNum, friendlyHostname);
			}
			policySOSPointer = configSOS->policies;
			while (policySOSPointer != 0)
			{
				ruleSOSPointer = policySOSPointer->policyRules;
				while (ruleSOSPointer != 0)
				{
					if ((ruleSOSPointer->permit == sos_permit_reject) && (ruleSOSPointer->disabled == false))
					{
						if (policyIssues > 1)
						{
							fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
							if (policySOSPointer->global == true)
								fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
							else
							{
								outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
								fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
								outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
								fprintf(reportFile, "%s%s", tempString, tablerow_mid);
							}
							fprintf(reportFile, "%s%s", report_policy_table8, tablerow_end);
						}
						else
						{
							if (policySOSPointer->global == true)
								sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs9);
							else
								sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs9);
						}
					}
					else if ((ruleSOSPointer->permit == sos_permit_permit) && (ruleSOSPointer->disabled == false))
					{
						destInt = 0;
						sourceInt = 0;
						serviceInt = 0;
						listSOSSourcePointer = ruleSOSPointer->source;
						while (listSOSSourcePointer != 0)
						{
							if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
								sourceInt++;
							listSOSSourcePointer = listSOSSourcePointer->next;
						}
						listSOSSourcePointer = ruleSOSPointer->dest;
						while (listSOSSourcePointer != 0)
						{
							if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
								destInt++;
							listSOSSourcePointer = listSOSSourcePointer->next;
						}
						listSOSSourcePointer = ruleSOSPointer->service;
						while (listSOSSourcePointer != 0)
						{
							if (strcasecmp(listSOSSourcePointer->name, "Any") == 0)
								serviceInt++;
							listSOSSourcePointer = listSOSSourcePointer->next;
						}

						// Output issue
						if ((sourceInt > 0) && (destInt > 0) && (serviceInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table4, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs5);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs5);
							}
						}
						else if ((sourceInt > 0) && (destInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table2, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs3);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs3);
							}
						}
						else if ((sourceInt > 0) && (serviceInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table3, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs4);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs4);
							}
						}
						else if (sourceInt > 0)
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table1, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs2);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs2);
							}
						}
						else if ((destInt > 0) && (serviceInt > 0))
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table6, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs7);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs7);
							}
						}
						else if (destInt > 0)
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table5, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs6);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs6);
							}
						}
						else if (serviceInt > 0)
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table7, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "ID %d in the Global policy list %s", ruleSOSPointer->id, report_policy_obs8);
								else
									sprintf(tempString, "ID %d in the zone %s to zone %s policy list %s", ruleSOSPointer->id, policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs8);
							}
						}
						if ((policyIssues == 1) && ((sourceInt > 0) || (destInt > 0) || (serviceInt > 0)))
						{
							outputFriendly(tempString, tempString2, maxSize, reportFormat);
							fprintf(reportFile, "%s", tempString2);
							output_newPara(reportFile, reportFormat);
						}
					}
					if (ruleSOSPointer->next == 0)
					{
						if ((ruleSOSPointer->permit != sos_permit_deny) || (ruleSOSPointer->disabled == true) || (ruleSOSPointer->log == false) || (strcasecmp(ruleSOSPointer->source->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->dest->name, "Any") != 0) || (strcasecmp(ruleSOSPointer->service->name, "ANY") != 0))
						{
							if (policyIssues > 1)
							{
								fprintf(reportFile, "%s %s", tablerow_start, tablerow_mid);
								if (policySOSPointer->global == true)
									fprintf(reportFile, "Yes%s %s %s", tablerow_mid, tablerow_mid, tablerow_mid);
								else
								{
									outputFriendly(policySOSPointer->fromZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "No%s%s%s", tablerow_mid, tempString, tablerow_mid);
									outputFriendly(policySOSPointer->toZone, tempString, maxSize, reportFormat);
									fprintf(reportFile, "%s%s", tempString, tablerow_mid);
								}
								fprintf(reportFile, "%s%s", report_policy_table9, tablerow_end);
							}
							else
							{
								if (policySOSPointer->global == true)
									sprintf(tempString, "The Global policy list %s", report_policy_obs10);
								else
									sprintf(tempString, "The zone %s to zone %s policy list %s", policySOSPointer->fromZone, policySOSPointer->toZone, report_policy_obs10);
							}
						}
					}
					ruleSOSPointer = ruleSOSPointer->next;
				}
				policySOSPointer = policySOSPointer->next;
			}
			if (policyIssues > 1)
			{
				output_table(reportFile, reportFormat, false, false, report_policy_table, &report_policy_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			output_endsection(reportFile, reportFormat, section_obs);
			if (rejectIssue == true)
				output_parseText(report_policy_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 2, 0);
			else
				output_parseText(report_policy_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 1, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_policy_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			if (denyIssue == true)
				output_parseText(report_accesslist_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 2, 0);
			else
				output_parseText(report_accesslist_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 1, 0);
		}

		// Timeout
		if (timeoutIssues > 0)
		{
			output_parseTitle(report_timeout_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
			output_parseText(report_timeout_obs1, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, timeoutIssues, tableNum);
			if ((connectionTimeout % 60) == 0)
			{
				numToWord(reportFile, (connectionTimeout / 60));
				fprintf(reportFile, " minutes");
			}
			else
			{
				numToWord(reportFile, connectionTimeout / 60);
				fprintf(reportFile, " minutes and ");
				numToWord(reportFile, connectionTimeout % 60);
				fprintf(reportFile, " seconds");
			}
			output_parseText(report_timeout_obs2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, timeoutIssues, tableNum);
			if (timeoutIssues > 1)
			{
				output_table(reportFile, reportFormat, true, false, report_timeout_table, &report_timeout_heading, tableNum, friendlyHostname);
				if (configSOS->admin != 0)
				{
					if ((configSOS->admin->authTimeout * 60) > connectionTimeout)
					{
						fprintf(reportFile, "%sWeb Administration%s", tablerow_start, tablerow_mid);
						numToWord(reportFile, configSOS->admin->authTimeout);
						fprintf(reportFile, " minutes%s", tablerow_end);
					}
				}
				authServerPointer = configSOS->authServer;
				while (authServerPointer != 0)
				{
					if ((authServerPointer->timeout * 60 ) > connectionTimeout)
					{
						switch (authServerPointer->type)
						{
							case authtype_builtin:
								fprintf(reportFile, "%sBuiltin Authentication%s", tablerow_start, tablerow_mid);
								break;
							case authtype_radius:
								fprintf(reportFile, "%sRADIUS Authentication%s", tablerow_start, tablerow_mid);
								break;
							case authtype_ldap:
								fprintf(reportFile, "%sLDAP Authentication%s", tablerow_start, tablerow_mid);
								break;
							case authtype_securid:
								fprintf(reportFile, "%sSecurId Authentication%s", tablerow_start, tablerow_mid);
								break;
						}
						numToWord(reportFile, authServerPointer->timeout);
						fprintf(reportFile, " minutes%s", tablerow_end);
					}
					authServerPointer = authServerPointer->next;
				}
				output_table(reportFile, reportFormat, false, false, report_timeout_table, &report_timeout_heading, tableNum, friendlyHostname);
				tableNum++;
			}
			else
			{
				if (configSOS->admin != 0)
				{
					if ((configSOS->admin->authTimeout * 60) > connectionTimeout)
					{
						fprintf(reportFile, "Web Administration had a timeout of ");
						numToWord(reportFile, configSOS->admin->authTimeout);
						fprintf(reportFile, " minutes.");
					}
				}
				authServerPointer = configSOS->authServer;
				while (authServerPointer != 0)
				{
					if ((authServerPointer->timeout * 60 ) > connectionTimeout)
					{
						switch (authServerPointer->type)
						{
							case authtype_builtin:
								fprintf(reportFile, "Builtin Authentication had a timeout of ");
								break;
							case authtype_radius:
								fprintf(reportFile, "RADIUS Authentication had a timeout of ");
								break;
							case authtype_ldap:
								fprintf(reportFile, "LDAP Authentication had a timeout of ");
								break;
							case authtype_securid:
								fprintf(reportFile, "SecurId Authentication had a timeout of ");
								break;
						}
						numToWord(reportFile, authServerPointer->timeout);
						fprintf(reportFile, " minutes.");
					}
					authServerPointer = authServerPointer->next;
				}
				output_newPara(reportFile, reportFormat);
			}
			output_endsection(reportFile, reportFormat, section_obs);
			output_parseText(report_timeout_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_high, 0, 0);
			output_endsection(reportFile, reportFormat, section_imp);
			output_parseText(report_timeout_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_difficult, 0, 0);
			output_endsection(reportFile, reportFormat, section_ease);
			output_parseText(report_timeout_rec1, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
			if ((connectionTimeout % 60) == 0)
			{
				numToWord(reportFile, (connectionTimeout / 60));
				fprintf(reportFile, " minutes");
			}
			else
			{
				numToWord(reportFile, connectionTimeout / 60);
				fprintf(reportFile, " minutes and ");
				numToWord(reportFile, (connectionTimeout % 60));
				fprintf(reportFile, " seconds");
			}
			output_parseText(report_timeout_rec2, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
			output_endsection(reportFile, reportFormat, section_rec);
		}

		// Administrative HTTP Redirect
		if (configSOS->admin != 0)
		{
			if (configSOS->admin->httpRedirect != true)
			{
				output_parseTitle(report_httpred_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
				output_parseText(report_httpred_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_obs);
				output_parseText(report_httpred_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
				output_endsection(reportFile, reportFormat, section_imp);
				output_parseText(report_httpred_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_trivial, 0, 0);
				output_endsection(reportFile, reportFormat, section_ease);
				output_parseText(report_httpred_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_rec);
			}
		}

		// Management IP
		if (configSOS->admin != 0)
		{
			if (strcmp(configSOS->admin->adminIP, "0.0.0.0") == 0)
			{
				output_parseTitle(report_mngrip_title, reportFile, reportFormat, friendlyHostname, rate_med, section_security);
				output_parseText(report_mngrip_obs, reportFile, reportFormat, companyName, friendlyHostname, section_obs, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_obs);
				output_parseText(report_mngrip_impact, reportFile, reportFormat, companyName, friendlyHostname, section_imp, rate_med, 0, 0);
				output_endsection(reportFile, reportFormat, section_imp);
				output_parseText(report_mngrip_ease, reportFile, reportFormat, companyName, friendlyHostname, section_ease, rate_noease, 0, 0);
				output_endsection(reportFile, reportFormat, section_ease);
				output_parseText(report_mngrip_rec, reportFile, reportFormat, companyName, friendlyHostname, section_rec, rate_none, 0, 0);
				output_endsection(reportFile, reportFormat, section_rec);
			}
		}

		// End of security issues
		output_endsection(reportFile, reportFormat, section_end);
	}

	// --------------------
	// Configuration Report
	// --------------------

	// Output configuration section title
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<hr><h2>%s Device Configuration</h2>\n\n", friendlyHostname);
			break;
		case output_xml:
			fprintf(reportFile, " <section type=\"configuration\" title=\"%s Device Configuration\">\n", friendlyHostname);
			break;
		case output_text:
			fprintf(reportFile, "%s Device Configuration\n~~~~~~~~~~~~~~~~~~~~~~~~", friendlyHostname);
			for (tempInt = 0; tempInt < strlen(friendlyHostname); tempInt++)
			{
				fprintf(reportFile, "~");
			}
			fprintf(reportFile, "\n\n");
			break;
		case output_latex:
			fprintf(reportFile, "\\section[AUDIT_%s]\n  {Configuration of the Cisco Device %s}\n\n", friendlyHostname, friendlyHostname);
			break;
	}

	// Output report introduction
	output_parseTitle("Introduction", reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_parseText(report_config_screenos_intro, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);

	// ------------------------------------------------------
	// Get List of Configuration options (and generate index)
	// ------------------------------------------------------

	// Start of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "<ul>\n");
			break;
	}

	output_parseIndex(settings_general_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configSOS->admin != 0)
		output_parseIndex(settings_admin_title, reportFile, reportFormat, friendlyHostname, section_config);
	output_parseIndex(settings_services_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configSOS->authServer != 0)
		output_parseIndex(settings_authservers_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configSOS->sshEnabled == true)
		output_parseIndex(settings_ssh_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configSOS->interface !=0)
		output_parseIndex(settings_inter_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configSOS->policies != 0)
		output_parseIndex(settings_policies_title, reportFile, reportFormat, friendlyHostname, section_config);
	if (configSOS->nameList != 0)
		output_parseIndex(settings_namemap_title, reportFile, reportFormat, friendlyHostname, section_config);

	// End of index
	switch (reportFormat)
	{
		case output_html:
			fprintf(reportFile, "</ul><br>\n");
			break;
		case output_text:
			fprintf(reportFile, "\n\n");
			break;
	}
	output_endsection(reportFile, reportFormat, subsection_end);

	// General Settings
	output_parseTitle(settings_general_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	fprintf(reportFile, "%sHostname%s%s%s", tablerow_start, tablerow_mid, friendlyHostname, tablerow_end);
	output_table(reportFile, reportFormat, false, passwords, settings_general_table, &settings_general_heading, tableNum, friendlyHostname);
	tableNum++;

	// Administrative Settings
	if (configSOS->admin != 0)
	{
		output_parseTitle(settings_admin_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_admin_table, &settings_general_heading, tableNum, friendlyHostname);
		if (configSOS->admin->name[0] != 0)
		{
			outputFriendly(configSOS->admin->name, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sAdministrative User%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		if ((configSOS->admin->password[0] != 0) && (passwords == true))
		{
			outputFriendly(configSOS->admin->password, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%sAdministrative Password%s%s%s", tablerow_start, tablerow_mid, tempString, tablerow_end);
		}
		if (configSOS->admin->restrictLength == 0)
			fprintf(reportFile, "%sAdmin Password Length%sAny Length (upto 31 characters)%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sAdmin Password Length%s%d%s", tablerow_start, tablerow_mid, configSOS->admin->restrictLength, tablerow_end);
		fprintf(reportFile, "%sAdmin Login Attempts%s%d%s", tablerow_start, tablerow_mid, configSOS->admin->accessAttempts, tablerow_end);
		if (configSOS->admin->privilege == adminpriv_default)
			fprintf(reportFile, "%sAdmin Privilages%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configSOS->admin->privilege == adminpriv_rw)
			fprintf(reportFile, "%sAdmin Privilages%sRead / Write%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configSOS->admin->privilege == adminpriv_ext)
			fprintf(reportFile, "%sAdmin Privilages%sGet Externally%s", tablerow_start, tablerow_mid, tablerow_end);
		if (strcmp(configSOS->admin->adminIP, "0.0.0.0") == 0)
			fprintf(reportFile, "%sAdmin Management IP%sAny%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sAdmin Management IP%s%s %s%s", tablerow_start, tablerow_mid, configSOS->admin->adminIP, configSOS->admin->adminNetMask, tablerow_end);
		addAbbreviation("IP", false);
		if (configSOS->admin->consoleOnly == true)
			fprintf(reportFile, "%sConsole Only Administration%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sConsole Only Administration%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (configSOS->admin->authServer[0] != 0)
			fprintf(reportFile, "%sAuthentication Server%s%s%s", tablerow_start, tablerow_mid, configSOS->admin->authServer, tablerow_end);
		if (configSOS->admin->authTimeout == 0)
			fprintf(reportFile, "%sAdministration Timeout%sNone%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sAdministration Timeout%s%d mins%s", tablerow_start, tablerow_mid, configSOS->admin->authTimeout, tablerow_end);
		if (configSOS->admin->httpRedirect == true)
			fprintf(reportFile, "%sHTTP Redirection%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configSOS->admin->httpRedirect == false)
			fprintf(reportFile, "%sHTTP Redirection%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sHTTP Redirection%sUnconfigured%s", tablerow_start, tablerow_mid, tablerow_end);
		addAbbreviation("HTTP", false);
		if (configSOS->admin->mailAlert == true)
			fprintf(reportFile, "%sMail Alerts%sYes%s", tablerow_start, tablerow_mid, tablerow_end);
		else
			fprintf(reportFile, "%sMail Alerts%sNo%s", tablerow_start, tablerow_mid, tablerow_end);
		if (configSOS->admin->email[0] != 0)
			fprintf(reportFile, "%sMail Alerts Address%s%s%s", tablerow_start, tablerow_mid, configSOS->admin->email, tablerow_end);
		if (configSOS->admin->email1[0] != 0)
			fprintf(reportFile, "%sMail Alerts Address%s%s%s", tablerow_start, tablerow_mid, configSOS->admin->email1, tablerow_end);
		if (configSOS->admin->email2[0] != 0)
			fprintf(reportFile, "%sMail Alerts Address%s%s%s", tablerow_start, tablerow_mid, configSOS->admin->email2, tablerow_end);
		if (configSOS->admin->emailServer[0] != 0)
			fprintf(reportFile, "%sMail Server%s%s%s", tablerow_start, tablerow_mid, configSOS->admin->emailServer, tablerow_end);
		if (configSOS->admin->adminFormat == adminfmt_default)
			fprintf(reportFile, "%sConfiguration Format%sDefault%s", tablerow_start, tablerow_mid, tablerow_end);
		else if (configSOS->admin->adminFormat == adminfmt_dos)
		{
			addAbbreviation("DOS", false);
			fprintf(reportFile, "%sConfiguration Format%sDOS%s", tablerow_start, tablerow_mid, tablerow_end);
		}
		else if (configSOS->admin->adminFormat == adminfmt_unix)
			fprintf(reportFile, "%sConfiguration Format%sUnix%s", tablerow_start, tablerow_mid, tablerow_end);

		output_table(reportFile, reportFormat, false, passwords, settings_admin_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;

		if (configSOS->admin->users != 0)
		{
			output_table(reportFile, reportFormat, true, passwords, settings_admins_table, &settings_admins_heading, tableNum, friendlyHostname);
			userSOSPointer = configSOS->admin->users;
			while (userSOSPointer != 0)
			{
				outputFriendly(userSOSPointer->username, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
				if (passwords == true)
					fprintf(reportFile, "%s%s", userSOSPointer->password, tablerow_mid);
				switch (userSOSPointer->privilege)
				{
					case adminpriv_default:
						fprintf(reportFile, "Default%s", tablerow_mid);
						break;
					case adminpriv_all:
						fprintf(reportFile, "All%s", tablerow_mid);
						break;
					case adminpriv_read:
						fprintf(reportFile, "Read Only%s", tablerow_mid);
						break;
				}
				if (userSOSPointer->trustee == trustee_none)
					fprintf(reportFile, "None%s", tablerow_end);
				else if (userSOSPointer->trustee == trustee_modem)
					fprintf(reportFile, "Modem%s", tablerow_end);
				else if (userSOSPointer->trustee == trustee_inter)
					fprintf(reportFile, "Interface%s", tablerow_end);
				userSOSPointer = userSOSPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, settings_admins_table, &settings_admins_heading, tableNum, friendlyHostname);
			tableNum++;
		}
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// Services
	output_parseTitle(settings_services_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
	output_table(reportFile, reportFormat, true, passwords, settings_services_table, &report_services_heading, tableNum, friendlyHostname);
	if (configSOS->sshEnabled == true)
		fprintf(reportFile, "%sSSH%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
	else
		fprintf(reportFile, "%sSSH%sDisabled%s", tablerow_start, tablerow_mid, tablerow_end);
	output_table(reportFile, reportFormat, false, passwords, settings_services_table, &report_services_heading, tableNum, friendlyHostname);
	tableNum++;
	addAbbreviation("SSH", false);

	// Authentication Servers
	if (configSOS->authServer != 0)
	{
		output_parseTitle(settings_authservers_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_authservers_table, &settings_authservers_heading, tableNum, friendlyHostname);
		authServerPointer = configSOS->authServer;
		while (authServerPointer != 0)
		{
			fprintf(reportFile, "%s%d%s", tablerow_start, authServerPointer->id, tablerow_mid);
			outputFriendly(authServerPointer->name, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			switch (authServerPointer->type)
			{
				case authtype_builtin:
					fprintf(reportFile, "Built-in%s", tablerow_mid);
					break;
				case authtype_radius:
					fprintf(reportFile, "RADIUS%s", tablerow_mid);
					addAbbreviation("RADIUS", false);
					break;
				case authtype_ldap:
					fprintf(reportFile, "LDAP%s", tablerow_mid);
					addAbbreviation("LDAP", false);
					break;
				case authtype_securid:
					fprintf(reportFile, "SecurID%s", tablerow_mid);
					break;
			}
			outputFriendly(authServerPointer->serverName, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			if (authServerPointer->backup2[0] == 0)
			{
				outputFriendly(authServerPointer->backup1, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			}
			else
			{
				outputFriendly(authServerPointer->backup1, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s, ", tempString);
				outputFriendly(authServerPointer->backup2, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			}
			if (authServerPointer->timeout == 0)
				fprintf(reportFile, "None%s", tablerow_mid);
			else
				fprintf(reportFile, "%d mins%s", authServerPointer->timeout, tablerow_mid);
			if (authServerPointer->forcedTimeout == 0)
				fprintf(reportFile, "None%s", tablerow_mid);
			else
				fprintf(reportFile, "%d mins%s", authServerPointer->forcedTimeout, tablerow_mid);
			outputFriendly(authServerPointer->sourceInterface, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			if (passwords == true)
			{
				outputFriendly(authServerPointer->radiusSecret, tempString, maxSize, reportFormat);
				fprintf(reportFile, "%s", tempString);
			}
			fprintf(reportFile, "%s", tablerow_end);
			authServerPointer = authServerPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_authservers_table, &settings_authservers_heading, tableNum, friendlyHostname);
		tableNum++;
		output_endsection(reportFile, reportFormat, subsection_end);
	}

	// SSH
	if (configSOS->sshEnabled == true)
	{
		output_parseTitle(settings_ssh_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_ssh_table, &settings_general_heading, tableNum, friendlyHostname);
		fprintf(reportFile, "%sSSH Service%sEnabled%s", tablerow_start, tablerow_mid, tablerow_end);
		fprintf(reportFile, "%sSSH Protocol Version%s%d%s", tablerow_start, tablerow_mid, configSOS->sshVersion, tablerow_end);
		fprintf(reportFile, "%sSSH V1 Key Generation Time%s%d mins%s", tablerow_start, tablerow_mid, configSOS->sshKeyGenTime, tablerow_end);
		fprintf(reportFile, "%sSSH V2 Public Key%s%s%s", tablerow_start, tablerow_mid, configSOS->sshPublicKey, tablerow_end);
		output_table(reportFile, reportFormat, false, passwords, settings_ssh_table, &settings_general_heading, tableNum, friendlyHostname);
		tableNum++;
	}

	// Interfaces
	if (configSOS->interface !=0)
	{
		output_parseTitle(settings_inter_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_table(reportFile, reportFormat, true, passwords, settings_inter_table, &settings_interSOS_heading, tableNum, friendlyHostname);
		interfaceSOSPointer = configSOS->interface;
		while (interfaceSOSPointer != 0)
		{
			outputFriendly(interfaceSOSPointer->name, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s%s", tablerow_start, tempString, tablerow_mid);
			if (interfaceSOSPointer->enabled == true)
				fprintf(reportFile, "Yes%s", tablerow_mid);
			else
				fprintf(reportFile, "No%s", tablerow_mid);
			outputFriendly(interfaceSOSPointer->ipAddress, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			outputFriendly(interfaceSOSPointer->zone, tempString, maxSize, reportFormat);
			fprintf(reportFile, "%s%s", tempString, tablerow_mid);
			if (interfaceSOSPointer->manageIdent == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageIdent == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageMtrace == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageMtrace == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageNsmgmt == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageNsmgmt == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->managePing == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->managePing == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageSnmp == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageSnmp == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageSsh == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageSsh == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageSsl == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageSsl == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageTelnet == true)
				fprintf(reportFile, "On%s", tablerow_mid);
			else if (interfaceSOSPointer->manageTelnet == false)
				fprintf(reportFile, "Off%s", tablerow_mid);
			else
				fprintf(reportFile, "-%s", tablerow_mid);
			if (interfaceSOSPointer->manageWeb == true)
				fprintf(reportFile, "On%s", tablerow_end);
			else if (interfaceSOSPointer->manageWeb == false)
				fprintf(reportFile, "Off%s", tablerow_end);
			else
				fprintf(reportFile, "-%s", tablerow_end);
			interfaceSOSPointer = interfaceSOSPointer->next;
		}
		output_table(reportFile, reportFormat, false, passwords, settings_inter_table, &settings_interSOS_heading, tableNum, friendlyHostname);
		output_endsection(reportFile, reportFormat, subsection_end);
		addAbbreviation("SNMP", false);
		addAbbreviation("SSL", false);
	}

	// Policies
	if (configSOS->policies !=0)
	{
		output_parseTitle(settings_policies_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		output_parseText(settings_policies_text, reportFile, reportFormat, companyName, friendlyHostname, section_none, rate_none, 0, 0);
		policySOSPointer = configSOS->policies;
		while (policySOSPointer != 0)
		{
			if (policySOSPointer->global == true)
				sprintf(tempString2, "Global zone policy list");
			else if (policySOSPointer->name[0] == 0)
				sprintf(tempString2, "Zone %s to zone %s policy list", policySOSPointer->fromZone, policySOSPointer->toZone);
			else
				sprintf(tempString2, "%s policy list (zone %s to zone %s)", policySOSPointer->name, policySOSPointer->fromZone, policySOSPointer->toZone);
			outputFriendly(tempString2, tempString, maxSize, reportFormat);
			output_table(reportFile, reportFormat, true, passwords, tempString, &settings_policies_heading, tableNum, friendlyHostname);
			ruleSOSPointer = policySOSPointer->policyRules;
			while (ruleSOSPointer != 0)
			{
				listSOSSourcePointer = ruleSOSPointer->source;
				while (listSOSSourcePointer != 0)
				{
					listSOSDestPointer = ruleSOSPointer->dest;
					while (listSOSDestPointer != 0)
					{
						listSOSServicePointer = ruleSOSPointer->service;
						while (listSOSServicePointer != 0)
						{
							fprintf(reportFile, "%s%d%s", tablerow_start, ruleSOSPointer->id, tablerow_mid);
							switch (ruleSOSPointer->permit)
							{
								case sos_permit_permit:
									fprintf(reportFile, "Permit%s", tablerow_mid);
									break;
								case sos_permit_deny:
									fprintf(reportFile, "Deny%s", tablerow_mid);
									break;
								case sos_permit_reject:
									fprintf(reportFile, "Reject%s", tablerow_mid);
									break;
							}
							if (ruleSOSPointer->disabled == true)
								fprintf(reportFile, "Yes%s", tablerow_mid);
							else
								fprintf(reportFile, "No%s", tablerow_mid);
							fprintf(reportFile, "%s%s", listSOSSourcePointer->name, tablerow_mid);
							fprintf(reportFile, "%s%s", listSOSDestPointer->name, tablerow_mid);
							fprintf(reportFile, "%s%s", listSOSServicePointer->name, tablerow_mid);
							addAbbreviation(listSOSServicePointer->name, false);
							if (ruleSOSPointer->log == true)
								fprintf(reportFile, "Yes%s", tablerow_end);
							else
								fprintf(reportFile, "No%s", tablerow_end);
							listSOSServicePointer = listSOSServicePointer->next;
						}
						listSOSDestPointer = listSOSDestPointer->next;
					}
					listSOSSourcePointer = listSOSSourcePointer->next;
				}
				ruleSOSPointer = ruleSOSPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, tempString, &settings_policies_heading, tableNum, friendlyHostname);
			tableNum++;
			policySOSPointer = policySOSPointer->next;
		}
	}

	if (configSOS->nameList != 0)
	{
		output_parseTitle(settings_namemap_title, reportFile, reportFormat, friendlyHostname, rate_none, section_config);
		addressSOSPointer = configSOS->nameList;
		while (addressSOSPointer != 0)
		{
			sprintf(tempString2, "Zone %s name mappings", addressSOSPointer->zone);
			outputFriendly(tempString2, tempString, maxSize, reportFormat);
			output_table(reportFile, reportFormat, true, passwords, tempString, &settings_namemapSOS_heading, tableNum, friendlyHostname);
			mappingSOSPointer = addressSOSPointer->mapping;
			while (mappingSOSPointer != 0)
			{
				outputFriendly(mappingSOSPointer->name, tempString2, maxSize, reportFormat);
				fprintf(reportFile, "%s%s%s", tablerow_start, tempString2, tablerow_mid);
				outputFriendly(mappingSOSPointer->address, tempString2, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString2, tablerow_mid);
				outputFriendly(mappingSOSPointer->netMask, tempString2, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString2, tablerow_mid);
				outputFriendly(mappingSOSPointer->comment, tempString2, maxSize, reportFormat);
				fprintf(reportFile, "%s%s", tempString2, tablerow_end);
				mappingSOSPointer = mappingSOSPointer->next;
			}
			output_table(reportFile, reportFormat, false, passwords, tempString, &settings_namemapSOS_heading, tableNum, friendlyHostname);
			tableNum++;
			addressSOSPointer = addressSOSPointer->next;
		}
	}

	// End of configuration section
	output_endsection(reportFile, reportFormat, section_end);

	// Appendix
	if (appendix == true)
		generateAppendix(reportFile, reportFormat, friendlyHostname, tableNum, abs, commonPorts, glossary);

	// Output end of file stuff
	output_fileend(reportFormat, reportFile);
}

